@@ -1,14 +1,13 @@
-/*  $Id: his.c,v 1.33.2.4 1998/12/26 16:16:10 kondou Exp $
+/*  $Id: his.c,v 1.43 2000/01/07 16:48:19 kondou Exp $
 **
 **  History file routines.
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include <netinet/in.h>
-#include "configdata.h"
+#include "config.h"
 #include "clibrary.h"
-#include "innd.h"
+#include <netinet/in.h>
+
 #include "dbz.h"
+#include "innd.h"
 
 typedef struct __HISCACHE {
     HASH	Hash;	/* Hash value of the message-id using Hash() */
@@ -86,7 +85,7 @@ void HISsetup(void)
 	   for files freopen()-ed in append mode according to POSIX 1003.1.
 	   ftell() is used later on to determine a new article's offset
 	   in the history file. Fopen() uses freopen() internally. */
-	if (fseek(HISwritefp, 0L, SEEK_END) == -1) {
+	if (fseek(HISwritefp, (OFFSET_T)0, SEEK_END) == -1) {
 	    syslog(L_FATAL, "cant fseek to end of %s %m", HIShistpath);
 	    exit(1);
 	}
@@ -105,7 +104,7 @@ void HISsetup(void)
 #ifdef	DO_TAGGED_HASH
 	opt.pag_incore = INCORE_MMAP;
 #else
-	opt.idx_incore = INCORE_NO;
+	opt.pag_incore = INCORE_NO;
 	opt.exists_incore = HISincore ? INCORE_MMAP : INCORE_NO;
 #endif
 	dbzsetoptions(opt);
@@ -183,17 +182,13 @@ void HISclose(void)
 /*
 **  Get the list of files under which a Message-ID is stored.
 */
-char *HISfilesfor(const HASH MessageID)
+TOKEN *HISfilesfor(const HASH MessageID)
 {
     static BUFFER	Files;
-    char		*dest;
     OFFSET_T		offset;
     char	        *p;
     int	                i;
-#ifndef DO_TAGGED_HASH
-    idxrec		ionevalue;
-    idxrecext		iextvalue;
-#endif
+    static TOKEN	token;
 
     TMRstart(TMR_HISGREP);
     
@@ -201,26 +196,10 @@ char *HISfilesfor(const HASH MessageID)
     	return NULL;
     
     /* Get the seek value into the history file. */
-#ifdef	DO_TAGGED_HASH
-    if ((offset = dbzfetch(MessageID)) < 0) {
+    if (!dbzfetch(MessageID, &offset)) {
 	TMRstop(TMR_HISGREP);
 	return NULL;
     }
-#else
-    if (innconf->extendeddbz) {
-	if (!dbzfetch(MessageID, &iextvalue)) {
-	    TMRstop(TMR_HISGREP);
-	    return NULL;
-	}
-	offset = iextvalue.offset[HISTOFFSET];
-    } else {
-	if (!dbzfetch(MessageID, &ionevalue)) {
-	    TMRstop(TMR_HISGREP);
-	    return NULL;
-	}
-	offset = ionevalue.offset;
-    }
-#endif
     TMRstop(TMR_HISGREP);
 
     /* Get space. */
@@ -258,12 +237,11 @@ char *HISfilesfor(const HASH MessageID)
 	return NULL;
     if ((p = strchr(p + 1, HIS_FIELDSEP)) == NULL)
 	return NULL;
-
-    /* Translate newsgroup separators to slashes, return the fieldstart. */
-    for (dest = ++p; *p; p++)
-	if (*p == '.')
-	    *p = '/';
-    return dest;
+    if (!IsToken(++p)) {
+	return NULL;
+    }
+    token = TextToToken(p);
+    return &token;
 }
 
 
@@ -304,27 +282,6 @@ BOOL HIShavearticle(const HASH MessageID)
 }
 
 
-/*
-**  Turn a history filename entry from slashes to dots.  It's a pity
-**  we have to do this.
-*/
-STATIC void HISslashify(char *p)
-{
-    char	        *last;
-
-    for (last = NULL; *p; p++) {
-	if (*p == '/') {
-	    *p = '.';
-	    last = p;
-	}
-	else if (*p == ' ' && last != NULL)
-	    *last = '/';
-    }
-    if (last)
-	*last = '/';
-}
-
-
 /*
 **  Write a history entry.
 */
@@ -333,64 +290,30 @@ BOOL HISwrite(const ARTDATA *Data, const HASH hash, char *paths, TOKEN *token)
     static char		NOPATHS[] = "";
     OFFSET_T		offset;
     int			i;
-#ifndef DO_TAGGED_HASH
-    void		*ivalue;
-    idxrec		ionevalue;
-    idxrecext		iextvalue;
-#endif
     
     if (HISwritefp == NULL)
         return FALSE;
 
     TMRstart(TMR_HISWRITE);
     if (paths != NULL && paths[0] != '\0') {
-	if (!innconf->storageapi) HISslashify(paths);
+	/* if (!innconf->storageapi) HISslashify(paths); */
     } else
 	paths = NOPATHS;
 
     offset = ftell(HISwritefp);
-    if (innconf->storageapi) {
-	if (Data->Expires > 0)
-	    i = fprintf(HISwritefp, "[%s]%c%lu%c%lu%c%lu%c%s\n",
-			HashToText(hash), HIS_FIELDSEP,
-			(unsigned long)Data->Arrived, HIS_SUBFIELDSEP,
-			(unsigned long)Data->Expires, HIS_SUBFIELDSEP,
-			(unsigned long)Data->Posted, HIS_FIELDSEP, paths);
-	else
-	    i = fprintf(HISwritefp, "[%s]%c%lu%c%s%c%lu%c%s\n",
-		        HashToText(hash), HIS_FIELDSEP,
-			(unsigned long)Data->Arrived, HIS_SUBFIELDSEP,
-			HIS_NOEXP, HIS_SUBFIELDSEP,
-			(unsigned long)Data->Posted, HIS_FIELDSEP, paths);
-    } else {
-	if (innconf->storemsgid) {
-	    if (Data->Expires > 0)
-		i = fprintf(HISwritefp, "%s%c%lu%c%lu%c%lu%c%s\n",
-			Data->MessageID, HIS_FIELDSEP,
-			(unsigned long)Data->Arrived, HIS_SUBFIELDSEP,
-			(unsigned long)Data->Expires, HIS_SUBFIELDSEP,
-			(unsigned long)Data->Posted, HIS_FIELDSEP, paths);
-	    else
-		i = fprintf(HISwritefp, "%s%c%lu%c%s%c%lu%c%s\n",
-			Data->MessageID, HIS_FIELDSEP,
-			(unsigned long)Data->Arrived, HIS_SUBFIELDSEP,
-			HIS_NOEXP, HIS_SUBFIELDSEP,
-			(unsigned long)Data->Posted, HIS_FIELDSEP, paths);
-	} else {
-	    if (Data->Expires > 0)
-		i = fprintf(HISwritefp, "[%s]%c%lu%c%lu%c%lu%c%s\n",
-			HashToText(hash), HIS_FIELDSEP,
-			(unsigned long)Data->Arrived, HIS_SUBFIELDSEP,
-			(unsigned long)Data->Expires, HIS_SUBFIELDSEP,
-			(unsigned long)Data->Posted, HIS_FIELDSEP, paths);
-	    else
-		i = fprintf(HISwritefp, "[%s]%c%lu%c%s%c%lu%c%s\n",
-			HashToText(hash), HIS_FIELDSEP,
-			(unsigned long)Data->Arrived, HIS_SUBFIELDSEP,
-			HIS_NOEXP, HIS_SUBFIELDSEP,
-			(unsigned long)Data->Posted, HIS_FIELDSEP, paths);
-	}
-    }
+    if (Data->Expires > 0)
+	i = fprintf(HISwritefp, "[%s]%c%lu%c%lu%c%lu%c%s\n",
+		    HashToText(hash), HIS_FIELDSEP,
+		    (unsigned long)Data->Arrived, HIS_SUBFIELDSEP,
+		    (unsigned long)Data->Expires, HIS_SUBFIELDSEP,
+		    (unsigned long)Data->Posted, HIS_FIELDSEP, paths);
+    else
+	i = fprintf(HISwritefp, "[%s]%c%lu%c%s%c%lu%c%s\n",
+		    HashToText(hash), HIS_FIELDSEP,
+		    (unsigned long)Data->Arrived, HIS_SUBFIELDSEP,
+		    HIS_NOEXP, HIS_SUBFIELDSEP,
+		    (unsigned long)Data->Posted, HIS_FIELDSEP, paths);
+
     if (i == EOF || fflush(HISwritefp) == EOF) {
 	/* The history line is now an orphan... */
 	i = errno;
@@ -401,31 +324,14 @@ BOOL HISwrite(const ARTDATA *Data, const HASH hash, char *paths, TOKEN *token)
     }
 
     /* Set up the database values and write them. */
-#ifdef	DO_TAGGED_HASH
     if (dbzstore(hash, offset) == DBZSTORE_ERROR) {
 	i = errno;
-	syslog(L_ERROR, "%s cant dbzstore %m", LogName);
+	syslog(L_ERROR, "%s cant dbzstore [%s]@%lu: %m", LogName,
+	       HashToText(hash), offset);
 	IOError("history database", i);
 	TMRstop(TMR_HISWRITE);
 	return FALSE;
     }
-#else
-    if (innconf->extendeddbz) {
-        iextvalue.offset[HISTOFFSET] = offset;
-        OVERsetoffset(token, &iextvalue.offset[OVEROFFSET], &iextvalue.overindex, &iextvalue.overlen);
-        ivalue = (void *)&iextvalue;
-    } else {
-        ionevalue.offset = offset;
-        ivalue = (void *)&ionevalue;
-    }
-    if (dbzstore(hash, ivalue) == DBZSTORE_ERROR) {
-	i = errno;
-	syslog(L_ERROR, "%s cant dbzstore %m", LogName);
-	IOError("history database", i);
-	TMRstop(TMR_HISWRITE);
-	return FALSE;
-    }
-#endif
     HIScacheadd(hash, TRUE);
     TMRstop(TMR_HISWRITE);
     
@@ -441,11 +347,6 @@ BOOL HISremember(const HASH hash)
 {
     OFFSET_T		offset;
     int			i;
-#ifndef DO_TAGGED_HASH
-    void		*ivalue;
-    idxrec		ionevalue;
-    idxrecext		iextvalue;
-#endif
 
     TMRstart(TMR_HISWRITE);
     
@@ -469,32 +370,14 @@ BOOL HISremember(const HASH hash)
     } 
 
     /* Set up the database values and write them. */
-#ifdef	DO_TAGGED_HASH
     if (dbzstore(hash, offset) == DBZSTORE_ERROR) {
 	i = errno;
-	syslog(L_ERROR, "%s cant dbzstore %m", LogName);
+	syslog(L_ERROR, "%s cant dbzstore [%s]@%lu: %m", LogName,
+	       HashToText(hash), offset);
 	IOError("history database", i);
 	TMRstop(TMR_HISWRITE);
 	return FALSE;
     }
-#else
-    if (innconf->extendeddbz) {
-        iextvalue.offset[HISTOFFSET] = offset;
-        iextvalue.offset[OVEROFFSET] = 0;
-	iextvalue.overindex = OVER_NONE;
-        ivalue = (void *)&iextvalue;
-    } else {
-        ionevalue.offset = offset;
-        ivalue = (void *)&ionevalue;
-    }
-    if (dbzstore(hash, ivalue) == DBZSTORE_ERROR) {
-	i = errno;
-	syslog(L_ERROR, "%s cant dbzstore %m", LogName);
-	IOError("history database", i);
-	TMRstop(TMR_HISWRITE);
-	return FALSE;
-    }
-#endif
     HIScacheadd(hash, TRUE);
     TMRstop(TMR_HISWRITE);
     
