@@ -1,46 +1,45 @@
-/*  $Id: lc.c 9841 2015-05-02 16:27:37Z iulius $
+/*  $Id: lc.c,v 1.10 1999/11/21 00:15:44 rra Exp $
 **
 **  Routines for the local connect channel.  Create a Unix-domain stream
 **  socket that processes on the local server connect to.  Once the
 **  connection is set up, we speak NNTP.  The connect channel is used only
 **  by rnews to feed in articles from the UUCP sites.
 */
-
-#include "config.h"
+#include <stdio.h>
+#include <sys/types.h>
+#include <netinet/in.h>
+#include "configdata.h"
 #include "clibrary.h"
-
-#include "inn/innconf.h"
 #include "innd.h"
+#if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
+#include <sys/un.h>
 
-
-#ifdef HAVE_UNIX_DOMAIN_SOCKETS
-# include "portable/socket-unix.h"
-
-static char	*LCpath = NULL;
-static CHANNEL	*LCchan;
+STATIC char	*LCpath = NULL;
+STATIC CHANNEL	*LCchan;
 
 
 /*
 **  Read function.  Accept the connection and create an NNTP channel.
 */
-static void
-LCreader(CHANNEL *cp)
+STATIC FUNCTYPE
+LCreader(cp)
+    CHANNEL	*cp;
 {
     int		fd;
     CHANNEL	*new;
 
     if (cp != LCchan) {
-	syslog(L_ERROR, "%s internal LCreader wrong channel 0x%p not 0x%p",
-	    LogName, (void *)cp, (void *)LCchan);
+	syslog(L_ERROR, "%s internal LCreader wrong channel 0x%x not 0x%x",
+	    LogName, cp, LCchan);
 	return;
     }
 
-    if ((fd = accept(cp->fd, NULL, NULL)) < 0) {
+    if ((fd = accept(cp->fd, (struct sockaddr *)NULL, (ARGTYPE *)NULL)) < 0) {
 	syslog(L_ERROR, "%s cant accept CCreader %m", LogName);
 	return;
     }
-    if ((new = NCcreate(fd, false, true)) != NULL) {
-	memset( &new->Address, 0, sizeof( new->Address ) );
+    if ((new = NCcreate(fd, FALSE, TRUE)) != NULL) {
+	new->Address.s_addr = MyAddress.s_addr;
 	syslog(L_NOTICE, "%s connected %d", "localhost", new->fd);
 	NCwritereply(new, (char *)NCgreeting);
     }
@@ -50,27 +49,26 @@ LCreader(CHANNEL *cp)
 /*
 **  Write-done function.  Shouldn't happen.
 */
-static void
-LCwritedone(CHANNEL *unused UNUSED)
+STATIC FUNCTYPE
+LCwritedone()
 {
     syslog(L_ERROR, "%s internal LCwritedone", LogName);
 }
-
-#endif /* HAVE_UNIX_DOMAIN_SOCKETS */
+#endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
 
 
 /*
 **  Create the channel.
 */
 void
-LCsetup(void)
+LCsetup()
 {
 #if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
     int			i;
     struct sockaddr_un	server;
 
     if (LCpath == NULL)
-	LCpath = concatpath(innconf->pathrun, INN_PATH_NNTPCONNECT);
+	LCpath = COPY(cpcatpath(innconf->pathrun, _PATH_NNTPCONNECT));
     /* Remove old detritus. */
     if (unlink(LCpath) < 0 && errno != ENOENT) {
 	syslog(L_FATAL, "%s cant unlink %s %m", LogName, LCpath);
@@ -82,9 +80,9 @@ LCsetup(void)
 	syslog(L_FATAL, "%s cant socket %s %m", LogName, LCpath);
 	exit(1);
     }
-    memset(&server, 0, sizeof server);
+    (void)memset((POINTER)&server, 0, sizeof server);
     server.sun_family = AF_UNIX;
-    strlcpy(server.sun_path, LCpath, sizeof(server.sun_path));
+    (void)strcpy(server.sun_path, LCpath);
     if (bind(i, (struct sockaddr *) &server, SUN_LEN(&server)) < 0) {
 	syslog(L_FATAL, "%s cant bind %s %m", LogName, LCpath);
 	exit(1);
@@ -106,13 +104,13 @@ LCsetup(void)
 **  Cleanly shut down the channel.
 */
 void
-LCclose(void)
+LCclose()
 {
 #if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
     CHANclose(LCchan, CHANname(LCchan));
     LCchan = NULL;
     if (unlink(LCpath) < 0)
 	syslog(L_ERROR, "%s cant unlink %s %m", LogName, LCpath);
-    free(LCpath);
+    DISPOSE(LCpath);
 #endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
 }
