@@ -1,4 +1,4 @@
-/*  $Id: perl.c 10305 2018-12-02 14:21:56Z iulius $
+/*  $Id: perl.c,v 1.15 1999/11/19 09:00:04 rra Exp $
 **
 **  Perl filtering support for innd.
 **
@@ -14,10 +14,18 @@
 **
 **  Future work:
 **
+**   - What we're doing with Path headers right now doesn't work for folded
+**     headers.  It's also kind of gross.  There has to be a better way of
+**     handling this.
+**
 **   - The breakdown between this file, lib/perl.c, and nnrpd/perl.c should
 **     be rethought, ideally in the light of supporting multiple filters in
 **     different languages.
 **
+**   - We're still calling strlen() on artBody, which should be avoidable
+**     since we've already walked it several times.  We should just cache
+**     the length somewhere for speed.
+**
 **   - Variable and key names should be standardized between this and nnrpd.
 **
 **   - The XS code is still calling CC* functions.  The common code between
@@ -32,84 +40,93 @@
 #include "config.h"
 
 /* Skip this entire file if DO_PERL (./configure --with-perl) isn't set. */
-#if DO_PERL
+#if defined(DO_PERL)
 
 #include "clibrary.h"
-#include "inn/wire.h"
 #include "innd.h"
+#include "art.h"
+
+/* Linux doesn't have bool, yet sets _G_HAVE_BOOL to true.  Hello? */
+#ifdef DO_NEED_BOOL
+typedef int bool;
+#endif
 
 #include <EXTERN.h>
-#pragma GCC diagnostic ignored "-Wredundant-decls"
 #include <perl.h>
-#pragma GCC diagnostic warning "-Wredundant-decls"
 #include <XSUB.h>
 #include "ppport.h"
 
-#include "innperl.h"
-
-/* Prototypes for XS functions. */
-XS(XS_INN_addhist);
-XS(XS_INN_article);
-XS(XS_INN_cancel);
-XS(XS_INN_filesfor);
-XS(XS_INN_havehist);
-XS(XS_INN_head);
-XS(XS_INN_newsgroup);
+/* From lib/perl.c. */
+extern BOOL		PerlFilterActive;
 
+/* From art.c.  Ew.  Need header parsing that doesn't use globals. */
+extern ARTHEADER	ARTheaders[], *ARTheadersENDOF;
+extern char		*filterPath;
 
 /*
 **  Run an incoming article through the Perl article filter.  Returns NULL
 **  accept the article or a rejection message to reject it.
 */
 char *
-PLartfilter(const ARTDATA *data, char *artBody, long artLen, int lines)
+PLartfilter(char *artBody, int lines)
 {
     dSP;
-    const ARTHEADER * hp;
-    const HDRCONTENT *hc = data->HdrContent;
+    ARTHEADER * hp;
     HV *        hdr;
     CV *        filter;
-    int         i, rc;
+    int         rc;
     char *      p;
+    char        save = '\0';
+    static SV * body = NULL;
     static char buf[256];
-    bool        failure;
-    SV *        errsv;
 
     if (!PerlFilterActive) return NULL;
     filter = perl_get_cv("filter_art", 0);
     if (!filter) return NULL;
 
-    /* Create %hdr and stash a copy of every known header. */
+    /* Create %hdr and stash a copy of every known header.  Path has to be
+       handled separately since it's been munged by article processing. */
     hdr = perl_get_hv("hdr", 1);
-    for (i = 0 ; i < MAX_ARTHEADER ; i++) {
-	if (HDR_FOUND(i)) {
-	    hp = &ARTheaders[i];
-            (void) hv_store(hdr, (char *) hp->Name, hp->Size, newSVpv(HDR(i), 0), 0);
+    for (hp = ARTheaders; hp < ARTheadersENDOF; hp++) {
+        if (hp->Found && hp->Value && strcmp(hp->Name, "Path") != 0)
+            hv_store(hdr, (char *) hp->Name, strlen(hp->Name),
+                     newSVpv(hp->Value, 0), 0);
+    }
+    if (filterPath) {
+        p = strpbrk(filterPath, "\r\n");
+        if (p) {
+	    save = *p;
+	    *p = '\0';
 	}
+        hv_store(hdr, "Path", 4, newSVpv(filterPath, 0), 0);
+        if (p) *p = save;
     }
 
     /* Store the article body.  We don't want to make another copy of it,
-       since it could potentially be quite large.  In testing, this produced
-       a 17% speed improvement over making a copy of the article body
-       for a fairly heavy filter.
-       Available since Perl 5.7.1, newSVpvn_share allows avoiding such
-       a copy (getting round its use for older versions of Perl leads
-       to unreliable SV * bodies as for regexps).  And for Perl not to
-       compute a hash for artBody, we give it "42". */
+       since it could potentially be quite large.  Instead, stash the
+       pointer in the static SV * body.  We set LEN to 0 and inc the
+       refcount to tell Perl not to free it (either one should be enough).
+       Requires 5.004.  In testing, this produced a 17% speed improvement
+       over making a copy of the article body for a fairly heavy filter. */
     if (artBody) {
-#if (PERL_REVISION == 5) && ((PERL_VERSION < 7) || ((PERL_VERSION == 7) && (PERL_SUBVERSION < 1)))
-        (void) hv_store(hdr, "__BODY__", 8, newSVpv(artBody, artLen), 0);
-#else
-        (void) hv_store(hdr, "__BODY__", 8, newSVpvn_share(artBody, artLen, 42), 0);
-#endif /* Perl < 5.7.1 */
+        if (!body) {
+            body = newSV(0);
+            (void) SvUPGRADE(body, SVt_PV);
+        }
+        SvPVX(body) = artBody;
+        SvCUR_set(body, strlen(artBody));
+        SvLEN_set(body, 0);
+        SvPOK_on(body);
+        (void) SvREADONLY_on(body);
+        (void) SvREFCNT_inc(body);
+        hv_store(hdr, "__BODY__", 8, body, 0);
     }
 
-    (void) hv_store(hdr, "__LINES__", 9, newSViv(lines), 0);
+    hv_store(hdr, "__LINES__", 9, newSViv(lines), 0);
 
     ENTER;
     SAVETMPS;
     PUSHMARK(SP);
-    PUTBACK;
     rc = perl_call_sv((SV *) filter, G_EVAL|G_SCALAR|G_NOARGS);
     SPAGAIN;
 
@@ -117,36 +134,28 @@ PLartfilter(const ARTDATA *data, char *artBody, long artLen, int lines)
 
     /* Check $@, which will be set if the sub died. */
     buf[0] = '\0';
-    errsv = ERRSV;
-    if (SvTRUE(errsv)) {
-        failure = true;
-        syslog(L_ERROR, "Perl function filter_art died on article %s: %s",
-               HDR_FOUND(HDR__MESSAGE_ID) ? HDR(HDR__MESSAGE_ID) : "?",
-               SvPV(errsv, PL_na));
+    if (SvTRUE(ERRSV)) {
+        syslog(L_ERROR, "Perl function filter_art died: %s",
+               SvPV(ERRSV, PL_na));
         (void) POPs;
-    } else {
-        failure = false;
-        if (rc == 1) {
-            p = POPp;
-            if (p && *p)
-                strlcpy(buf, p, sizeof(buf));
+        PerlFilter(FALSE);
+    } else if (rc == 1) {
+        p = POPp;
+        if (p && *p) {
+            strncpy(buf, p, sizeof(buf) - 1);
+            buf[sizeof(buf) - 1] = '\0';
         }
     }
 
     PUTBACK;
     FREETMPS;
     LEAVE;
-
-    if (failure)
-        PerlFilter(false);
-
     return (buf[0] != '\0') ? buf : NULL;
 }
 
 
 /*
-**  Run an incoming message-ID from CHECK, IHAVE or TAKETHIS through
-**  the Perl filter.
+**  Run an incoming message ID from CHECK or IHAVE through the Perl filter.
 **  Returns NULL to accept the article or a rejection message to reject it.
 */
 char *
@@ -157,8 +166,6 @@ PLmidfilter(char *messageID)
     int         rc;
     char        *p;
     static char buf[256];
-    bool        failure;
-    SV *        errsv;
 
     if (!PerlFilterActive) return NULL;
     filter = perl_get_cv("filter_messageid", 0);
@@ -175,28 +182,22 @@ PLmidfilter(char *messageID)
 
     /* Check $@, which will be set if the sub died. */
     buf[0] = '\0';
-    errsv = ERRSV;
-    if (SvTRUE(errsv)) {
-        failure = true;
-        syslog(L_ERROR, "Perl function filter_messageid died on id %s: %s",
-               messageID, SvPV(errsv, PL_na));
+    if (SvTRUE(ERRSV)) {
+        syslog(L_ERROR, "Perl function filter_messageid died: %s",
+               SvPV(ERRSV, PL_na));
         (void) POPs;
-    } else {
-        failure = false;
-        if (rc == 1) {
-            p = POPp;
-            if (p && *p)
-                strlcpy(buf, p, sizeof(buf));
+        PerlFilter(FALSE);
+    } else if (rc == 1) {
+        p = POPp;
+        if (p && *p) {
+            strncpy(buf, p, sizeof(buf) - 1);
+            buf[sizeof(buf) - 1] = '\0';
         }
     }
-
+    
     PUTBACK;
     FREETMPS;
     LEAVE;
-
-    if (failure)
-        PerlFilter(false);
-
     return (buf[0] != '\0') ? buf : NULL;
 }
 
@@ -206,13 +207,11 @@ PLmidfilter(char *messageID)
 **  mode and the reason.
 */
 void
-PLmode(OPERATINGMODE CurrentMode, OPERATINGMODE NewMode, char *reason)
+PLmode(OPERATINGMODE Mode, OPERATINGMODE NewMode, char *reason)
 {
     dSP;
     HV          *mode;
     CV          *filter;
-    bool        failure;
-    SV *        errsv;
 
     filter = perl_get_cv("filter_mode", 0);
     if (!filter) return;
@@ -221,88 +220,31 @@ PLmode(OPERATINGMODE CurrentMode, OPERATINGMODE NewMode, char *reason)
        the reason in $mode{reason}. */
     mode = perl_get_hv("mode", 1);
 
-    if (CurrentMode == OMrunning)
-        (void) hv_store(mode, "Mode", 4, newSVpv("running", 7), 0);
-    if (CurrentMode == OMpaused)
-        (void) hv_store(mode, "Mode", 4, newSVpv("paused", 6), 0);
-    if (CurrentMode == OMthrottled)
-        (void) hv_store(mode, "Mode", 4, newSVpv("throttled", 9), 0);
-    if (CurrentMode == OMshutdown)
-        (void) hv_store(mode, "Mode", 4, newSVpv("shutdown", 8), 0);
+    if (Mode == OMrunning)
+        hv_store(mode, "Mode", 4, newSVpv("running", 0), 0);
+    if (Mode == OMpaused)
+        hv_store(mode, "Mode", 4, newSVpv("paused", 0), 0);
+    if (Mode == OMthrottled)
+        hv_store(mode, "Mode", 4, newSVpv("throttled", 0), 0);
 
     if (NewMode == OMrunning)
-        (void) hv_store(mode, "NewMode", 7, newSVpv("running", 7), 0);
+        hv_store(mode, "NewMode", 7, newSVpv("running", 0), 0);
     if (NewMode == OMpaused)
-        (void) hv_store(mode, "NewMode", 7, newSVpv("paused", 6), 0);
+        hv_store(mode, "NewMode", 7, newSVpv("paused", 0), 0);
     if (NewMode == OMthrottled)
-        (void) hv_store(mode, "NewMode", 7, newSVpv("throttled", 9), 0);
-    if (NewMode == OMshutdown)
-        (void) hv_store(mode, "NewMode", 7, newSVpv("shutdown", 8), 0);
+        hv_store(mode, "NewMode", 7, newSVpv("throttled", 0), 0);
 
-    (void) hv_store(mode, "reason", 6, newSVpv(reason, 0), 0);
+    hv_store(mode, "reason", 6, newSVpv(reason, 0), 0);
 
-    ENTER;
-    SAVETMPS;
     PUSHMARK(SP);
-    PUTBACK;
-
     perl_call_sv((SV *) filter, G_EVAL|G_DISCARD|G_NOARGS);
 
-    SPAGAIN;
-
     /* Check $@, which will be set if the sub died. */
-    errsv = ERRSV;
-    if (SvTRUE(errsv)) {
-        failure = true;
+    if (SvTRUE(ERRSV)) {
         syslog(L_ERROR, "Perl function filter_mode died: %s",
-                SvPV(errsv, PL_na));
+                SvPV(ERRSV, PL_na));
         (void) POPs;
-    } else {
-        failure = false;
-    }
-
-    hv_undef(mode);
-
-    PUTBACK;
-    FREETMPS;
-    LEAVE;
-
-    if (failure)
-        PerlFilter(false);
-}
-
-
-/*
-**  Called by CCmode, this returns the Perl filter statistics if a Perl
-**  function to generate such statistics has been defined, or NULL otherwise.
-**  If a string is returned, it's in newly allocated memory that must be freed
-**  by the caller.
-*/
-char *
-PLstats(void)
-{
-    dSP;
-    char *argv[] = { NULL };
-
-    if (perl_get_cv("filter_stats", false) == NULL)
-        return NULL;
-    else {
-        char *stats = NULL;
-        char *result;
-
-	ENTER;
-	SAVETMPS;
-        /* No need for PUSHMARK(SP) with call_argv(). */
-	perl_call_argv("filter_stats", G_EVAL | G_NOARGS, argv);
-	SPAGAIN;
-        result = POPp;
-        if (result != NULL && *result)
-            stats = xstrdup(result);
-	PUTBACK;
-	FREETMPS;
-	LEAVE;
-
-        return stats;
+        PerlFilter(FALSE);
     }
 }
 
@@ -329,9 +271,6 @@ XS(XS_INN_addhist)
     char        tbuff[32];
     char*       parambuf[6];
 
-    /* Suppress warnings for the mandatory XS argument. */
-    cv = cv;
-
     if (items < 1 || items > 5)
         croak("Usage INN::addhist(msgid,[arrival,articletime,expire,token])");
 
@@ -340,14 +279,14 @@ XS(XS_INN_addhist)
 
     /* If any of the times are missing, they should default to now. */
     if (i < 4) {
-        snprintf(tbuff, sizeof(tbuff), "%ld", (long) time(NULL));
+        sprintf(tbuff, "%ld", time((long *) 0));
         for (; i < 4; i++)
             parambuf[i] = tbuff;
     }
 
     /* The token defaults to an empty string. */
     if (i == 4)
-        parambuf[4] = (char *) "";
+        parambuf[4] = "";
 
     parambuf[5] = NULL;
 
@@ -369,31 +308,29 @@ XS(XS_INN_article)
 {
     dXSARGS;
     char *      msgid;
-    TOKEN       token;
+    TOKEN *     token;
     ARTHANDLE * art;
     char *      p;
-    size_t      len;
-
-    /* Suppress warnings for the mandatory XS argument. */
-    cv = cv;
+    int         len;
 
     if (items != 1)
 	croak("Usage: INN::article(msgid)");
 
     /* Get the article token from the message ID and the history file. */
     msgid = (char *) SvPV(ST(0), PL_na);
-    if (!HISlookup(History, msgid, NULL, NULL, NULL, &token)) XSRETURN_UNDEF;
+    token = HISfilesfor(HashMessageID(msgid));
+    if (token == NULL) XSRETURN_UNDEF;
 
     /* Retrieve the article and convert it from wire format. */
-    art = SMretrieve(token, RETR_ALL);
+    art = SMretrieve(*token, RETR_ALL);
     if (art == NULL) XSRETURN_UNDEF;
-    p = wire_to_native(art->data, art->len, &len);
+    p = FromWireFmt(art->data, art->len, &len);
     SMfreearticle(art);
 
     /* Push a copy of the article onto the Perl stack, free our temporary
        memory allocation, and return the article to Perl. */
     ST(0) = sv_2mortal(newSVpv(p, len));
-    free(p);
+    DISPOSE(p);
     XSRETURN(1);
 }
 
@@ -408,9 +345,6 @@ XS(XS_INN_cancel)
     char        *msgid;
     char        *parambuf[2];
 
-    /* Suppress warnings for the mandatory XS argument. */
-    cv = cv;
-
     if (items != 1)
         croak("Usage: INN::cancel(msgid)");
 
@@ -435,17 +369,15 @@ XS(XS_INN_filesfor)
 {
     dXSARGS;
     char        *msgid;
-    TOKEN       token;
-
-    /* Suppress warnings for the mandatory XS argument. */
-    cv = cv;
+    TOKEN       *token;
 
     if (items != 1)
         croak("Usage: INN::filesfor(msgid)");
 
     msgid = (char *) SvPV(ST(0), PL_na);
-    if (HISlookup(History, msgid, NULL, NULL, NULL, &token)) {
-        XSRETURN_PV(TokenToText(token));
+    token = HISfilesfor(HashMessageID(msgid));
+    if (token) {
+        XSRETURN_PV(TokenToText(*token));
     } else {
         XSRETURN_UNDEF;
     }
@@ -460,14 +392,11 @@ XS(XS_INN_havehist)
     dXSARGS;
     char        *msgid;
 
-    /* Suppress warnings for the mandatory XS argument. */
-    cv = cv;
-
     if (items != 1)
         croak("Usage: INN::havehist(msgid)");
 
     msgid = (char *) SvPV(ST(0), PL_na);
-    if (HIScheck(History, msgid))
+    if (HIShavearticle(HashMessageID(msgid)))
         XSRETURN_YES;
     else
         XSRETURN_NO;
@@ -483,31 +412,29 @@ XS(XS_INN_head)
 {
     dXSARGS;
     char *      msgid;
-    TOKEN       token;
+    TOKEN *     token;
     ARTHANDLE * art;
     char *      p;
-    size_t      len;
-
-    /* Suppress warnings for the mandatory XS argument. */
-    cv = cv;
+    int         len;
 
     if (items != 1)
         croak("Usage: INN::head(msgid)");
 
     /* Get the article token from the message ID and the history file. */
     msgid = (char *) SvPV(ST(0), PL_na);
-    if (!HISlookup(History, msgid, NULL, NULL, NULL, &token)) XSRETURN_UNDEF;
+    token = HISfilesfor(HashMessageID(msgid));
+    if (token == NULL) XSRETURN_UNDEF;
 
     /* Retrieve the article header and convert it from wire format. */
-    art = SMretrieve(token, RETR_HEAD);
+    art = SMretrieve(*token, RETR_HEAD);
     if (art == NULL) XSRETURN_UNDEF;
-    p = wire_to_native(art->data, art->len, &len);
+    p = FromWireFmt(art->data, art->len, &len);
     SMfreearticle(art);
 
     /* Push a copy of the article header onto the Perl stack, free our
        temporary memory allocation, and return the header to Perl. */
     ST(0) = sv_2mortal(newSVpv(p, len));
-    free(p);
+    DISPOSE(p);
     XSRETURN(1);
 }
 
@@ -524,9 +451,6 @@ XS(XS_INN_newsgroup)
     char *      end;
     int         size;
 
-    /* Suppress warnings for the mandatory XS argument. */
-    cv = cv;
-
     if (items != 1)
         croak("Usage: INN::newsgroup(group)");
     newsgroup = (char *) SvPV(ST(0), PL_na);
@@ -552,15 +476,15 @@ XS(XS_INN_newsgroup)
 **  Initialize the XS callbacks defined in this file.
 */
 void
-PLxsinit(void)
+PLxsinit()
 {
-    inn_newXS("INN::addhist", XS_INN_addhist, "perl.c");
-    inn_newXS("INN::article", XS_INN_article, "perl.c");
-    inn_newXS("INN::cancel", XS_INN_cancel, "perl.c");
-    inn_newXS("INN::havehist", XS_INN_havehist, "perl.c");
-    inn_newXS("INN::head", XS_INN_head, "perl.c");
-    inn_newXS("INN::newsgroup", XS_INN_newsgroup, "perl.c");
-    inn_newXS("INN::filesfor", XS_INN_filesfor, "perl.c");
+    newXS("INN::addhist", XS_INN_addhist, "perl.c");
+    newXS("INN::article", XS_INN_article, "perl.c");
+    newXS("INN::cancel", XS_INN_cancel, "perl.c");
+    newXS("INN::havehist", XS_INN_havehist, "perl.c");
+    newXS("INN::head", XS_INN_head, "perl.c");
+    newXS("INN::newsgroup", XS_INN_newsgroup, "perl.c");
+    newXS("INN::filesfor", XS_INN_filesfor, "perl.c");
 }
 
 #endif /* defined(DO_PERL) */
