@@ -1,19 +1,9 @@
-/* $Id: timer.c,v 1.9 1998/07/14 00:01:28 kondou Exp $
-**
-** Timer functions, to gather profiling data
+/* $Id: timer.c,v 1.13 2000/03/31 00:36:34 rra Exp $
 **
+** Timer functions, to gather profiling data.
 */
-
-#include <stdio.h>
-#include <string.h>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <netinet/in.h>
-
-#include "configdata.h"
 #include "clibrary.h"
 #include "innd.h"
-#include <syslog.h> 
 
 unsigned	start[TMR_MAX];
 unsigned	cumulative[TMR_MAX];
@@ -103,7 +93,9 @@ static void dosummary(unsigned secs)
 	case TMR_ARTCNCL:	str = "artcncl";  break;
 	case TMR_HISHAVE:	str = "hishave";  break;
 	case TMR_HISGREP:	str = "hisgrep";  break;
+	case TMR_OVERV:		str = "overv";  break;
 	case TMR_PERL:		str = "perl";  break;
+	case TMR_PYTHON:	str = "python";  break;
 	}
 	sprintf(buf, "%s %d(%d) ", str, cumulative[i], count[i]);
 	cumulative[i] = count[i] = 0;
@@ -113,18 +105,21 @@ static void dosummary(unsigned secs)
 }
 
 
-void TMRmainloophook(void)
+int
+TMRmainloophook(void)
 {
     unsigned now;
     
     if (!innconf->timer)
-	return;
+	return 0;
     now = gettime();
     
-    if (now - last_summary > (innconf->timer * 1000)) {
+    if (now - last_summary >= (innconf->timer * 1000)) {
 	dosummary(now - last_summary);
 	last_summary = now;
+        return 0;
     }
+    return innconf->timer - (now - last_summary) / 1000;
 }
 
 
