@@ -1,20 +1,50 @@
-/*  $Id: tape.c 9695 2014-09-20 05:53:22Z iulius $
-**
-**  The implementation of the innfeed Tape class.
-**
-**  Written by James Brister <brister@vix.com>
-**
-**  The implementation of the Tape class. Tapes are read-only or write-only
-**  files that are accessed sequentially. Their basic unit of i/o is an
-**  Article. Tapes work out of a single directory and manage all file names
-**  themselves.
-**
-**  Tapes will checkpoint themselves periodically so that when innfeed exits
-**  or crashes things can restart close to where they were last. The period
-**  checkpointing is handled entirely by the Tape class, but the checkpoint
-**  period needs to be set by some external user before the first tape is
-**  created.
-*/
+/* -*- c -*-
+ *
+ * Author:      James Brister <brister@vix.com> -- berkeley-unix --
+ * Start Date:  Sun Dec 31 18:52:04 1995
+ * Project:     INN (innfeed)
+ * File:        tape.c
+ * RCSId:       $Id: tape.c,v 1.13.2.1 2000/08/18 05:03:57 rra Exp $
+ *
+ * Copyright:   Copyright (c) 1996 by Internet Software Consortium
+ *
+ *              Permission to use, copy, modify, and distribute this
+ *              software for any purpose with or without fee is hereby
+ *              granted, provided that the above copyright notice and this
+ *              permission notice appear in all copies.
+ *
+ *              THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE
+ *              CONSORTIUM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
+ *              SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ *              MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET
+ *              SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT,
+ *              INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ *              WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
+ *              WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+ *              TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
+ *              USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * Description: The implementation of the Tape class. Tapes are read-only
+ *              or write-only files that are accessed sequentially. Their
+ *              basic unit of i/o is an Article. Tapes work out of a single
+ *              directory and manage all file names themselves.
+ *
+ *              Tapes will checkpoint themselves periodically so
+ *              that when innfeed exits or crashes things can
+ *              restart close to where they were last. The period
+ *              checkpointing is handled entirely by the Tape class,
+ *              but the checkpoint period needs to be set by some
+ *              external user before the first tape is created.
+ *
+ */
+
+ 
+#if ! defined (lint)
+static const char *rcsid = "$Id: tape.c,v 1.13.2.1 2000/08/18 05:03:57 rra Exp $" ;
+static void use_rcsid (const char *rid) {   /* Never called */
+  use_rcsid (rcsid) ; use_rcsid (rid) ;
+}
+#endif
 
 #include "innfeed.h"
 #include "config.h"
@@ -27,26 +57,35 @@
 #include <sys/stat.h>
 #include <syslog.h>
 
+#if defined (HAVE_DIRENT_H)
 #include <dirent.h>
 typedef struct dirent DIRENTRY ;
-
-#ifdef HAVE_SYS_TIME_H
-# include <sys/time.h>
+#else
+#include <sys/dir.h>
+typedef struct direct DIRENTRY ;
 #endif
-#include <time.h>
 
-#include "inn/innconf.h"
-#include "inn/messages.h"
-#include "inn/libinn.h"
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
+#include "libinn.h"
 
 #include "article.h"
 #include "configfile.h"
 #include "endpoint.h"
+#include "msgs.h"
 #include "tape.h"
 
 extern char *dflTapeDir;
 extern bool genHtml ;
-extern unsigned int hostHighwater;
 
 #if 0
 /* a structure for temporary storage of articles. */
@@ -86,7 +125,7 @@ struct tape_s
     /* the tape holds a small output queue in memory to avoid thrashing. */
     QueueElem head ;            
     QueueElem tail ;
-    unsigned int qLength ;             /* amount on the queue */
+    u_int qLength ;             /* amount on the queue */
 #endif
     
     long outputSize ;           /* the current size of the output file. */
@@ -136,8 +175,11 @@ static TimeoutId checkPtId ;
 static TimeoutId ckNewFileId ;
 
 /* number of seconds between tape checkpoints. */
-static unsigned int tapeCkPtPeriod ;
-static unsigned int tapeCkNewFilePeriod ;
+static u_int tapeCkPtPeriod ;
+static u_int tapeCkNewFilePeriod ;
+
+/* the callback ID of the tape rotation timer callback. */
+static TimeoutId rotateId ;     /* XXX */
 
 static time_t rotatePeriod = TAPE_ROTATE_PERIOD ;
 
@@ -155,7 +197,7 @@ static size_t activeTapeIdx ;
 static long defaultSizeLimit ;
 #endif
 
-unsigned int tapeHighwater ;
+u_int tapeHighwater ;
 
 bool debugShrinking = false ;
 
@@ -175,38 +217,31 @@ int tapeConfigLoadCbk (void *data)
 
   if (getString (topScope,"backlog-directory",&p,NO_INHERIT))
     {
-      dir = concatpath(innconf->pathspool, p);
+      dir = buildFilename(innconf->pathspool, p);
       free(p);
       if (tapeDirectory != NULL && strcmp (tapeDirectory,dir) != 0)
         {
-          warn ("ME config: cannot change backlog-directory of a running"
-                " process") ;
-          free (dir) ;
-          dir = xstrdup (tapeDirectory) ;
+          syslog (LOG_ERR,NO_CHANGE_BACKLOG) ;
+          FREE (dir) ;
+          dir = strdup (tapeDirectory) ;
         }
 
       if (!isDirectory (dir) && isDirectory (dflTapeDir))
         {
-          logOrPrint (LOG_ERR,fp,
-                      "ME config: definition of backlog-directory (%s) is a"
-                      " non-existant directory. Using %s",
-                      dir,dflTapeDir) ;
-          free (dir) ;
-          dir = xstrdup (dflTapeDir) ;
+          logOrPrint (LOG_ERR,fp,BAD_TAPEDIR_CHANGE,dir,dflTapeDir) ;
+          FREE (dir) ;
+          dir = strdup (dflTapeDir) ;
         }
       else if (!isDirectory (dir))
-        logAndExit (1,"ME config: no usable value for backlog-directory") ;
+        logAndExit (1,NO_TAPE_DIR) ;
     }
   else if (!isDirectory (dflTapeDir))
-  {
-    logAndExit (1,"ME config: no usable value for backlog-directory") ;
-    return -1; /* not reached */
-  }
+    logAndExit (1,NO_TAPE_DIR) ;
   else
-    dir = xstrdup (dflTapeDir) ;
+    dir = strdup (dflTapeDir) ;
   
   if (tapeDirectory != NULL)
-    free (tapeDirectory) ;
+    FREE (tapeDirectory) ;
   tapeDirectory = dir ;
 
 
@@ -216,16 +251,14 @@ int tapeConfigLoadCbk (void *data)
       if (iv < 0)
         {
           rval = 0 ;
-          logOrPrint (LOG_ERR,fp,
-                      "ME config: value of %s (%ld) in %s cannot be less"
-                      " than 0. Using %ld","backlog-highwater",
+          logOrPrint (LOG_ERR,fp,LESS_THAN_ZERO,"backlog-highwater",
                       iv,"global scope",(long)TAPE_HIGHWATER);
           iv = TAPE_HIGHWATER ;
         }
     }
   else
     iv = TAPE_HIGHWATER ;
-  tapeHighwater = (unsigned int) iv ;
+  tapeHighwater = (u_int) iv ;
 
 
   
@@ -234,17 +267,14 @@ int tapeConfigLoadCbk (void *data)
       if (iv < 0)
         {
           rval = 0 ;
-          logOrPrint (LOG_ERR,fp,
-                      "ME config: value of %s (%ld) in %s cannot be less"
-                      " than 0. Using %ld",
-                      "backlog-rotate-period",
+          logOrPrint (LOG_ERR,fp,LESS_THAN_ZERO,"backlog-rotate-period",
                       iv,"global scope",(long)TAPE_ROTATE_PERIOD);
           iv = TAPE_ROTATE_PERIOD ;
         }
     }
   else
     iv = TAPE_ROTATE_PERIOD ;
-  rotatePeriod = (unsigned int) iv ;
+  rotatePeriod = (u_int) iv ;
 
 
   if (getInteger (topScope,"backlog-ckpt-period",&iv,NO_INHERIT))
@@ -252,17 +282,14 @@ int tapeConfigLoadCbk (void *data)
       if (iv < 0)
         {
           rval = 0 ;
-          logOrPrint (LOG_ERR,fp,
-                      "ME config: value of %s (%ld) in %s cannot be less"
-                      " than 0. Using %ld",
-                      "backlog-ckpt-period",iv,
+          logOrPrint (LOG_ERR,fp,LESS_THAN_ZERO,"backlog-ckpt-period",iv,
                       "global scope",(long)TAPE_CHECKPOINT_PERIOD);
           iv = TAPE_CHECKPOINT_PERIOD ;
         }
     }
   else
     iv = TAPE_CHECKPOINT_PERIOD ;
-  tapeCkPtPeriod = (unsigned int) iv ;
+  tapeCkPtPeriod = (u_int) iv ;
 
 
   if (getInteger (topScope,"backlog-newfile-period",&iv,NO_INHERIT))
@@ -270,17 +297,14 @@ int tapeConfigLoadCbk (void *data)
       if (iv < 0)
         {
           rval = 0 ;
-          logOrPrint (LOG_ERR,fp,
-                      "ME config: value of %s (%ld) in %s cannot be less"
-                      " than 0. Using %ld",
-                      "backlog-newfile-period",
+          logOrPrint (LOG_ERR,fp,LESS_THAN_ZERO,"backlog-newfile-period",
                       iv,"global scope",(long)TAPE_NEWFILE_PERIOD);
           iv = TAPE_NEWFILE_PERIOD ;
         }
     }
   else
     iv = TAPE_NEWFILE_PERIOD ;
-  tapeCkNewFilePeriod = (unsigned int) iv ;
+  tapeCkNewFilePeriod = (u_int) iv ;
 
 
   if (getBool (topScope,"debug-shrinking",&bv,NO_INHERIT))
@@ -301,7 +325,7 @@ int tapeConfigLoadCbk (void *data)
 static bool inited = false ;
 Tape newTape (const char *peerName, bool dontRotate)
 {
-  Tape nt = xmalloc (sizeof(struct tape_s)) ;
+  Tape nt = ALLOC (struct tape_s, 1) ;
   size_t pLen = strlen (peerName) ;
   size_t dLen = strlen (tapeDirectory) ;
 
@@ -322,29 +346,33 @@ Tape newTape (const char *peerName, bool dontRotate)
   if (endsIn (peerName,LOCK_TAIL))
     die ("Sorry, can't have a peer name ending in \"%s\"",LOCK_TAIL) ;
 
-  nt->peerName = xstrdup (peerName) ;
+  nt->peerName = strdup (peerName) ;
   
-  nt->handFilename = xmalloc (pLen + dLen + 2) ;
+  nt->handFilename = MALLOC (pLen + dLen + 2) ;
+  ASSERT (nt->handFilename != NULL) ;
   sprintf (nt->handFilename,"%s/%s",tapeDirectory,peerName) ;
 
-  nt->lockFilename = xmalloc (pLen + dLen + strlen(LOCK_TAIL) + 2) ;
+  nt->lockFilename = MALLOC (pLen + dLen + strlen(LOCK_TAIL) + 2) ;
+  ASSERT (nt->lockFilename != NULL) ;
   sprintf (nt->lockFilename,"%s/%s%s",tapeDirectory,peerName,LOCK_TAIL) ;
 
-  nt->inputFilename = xmalloc (pLen + dLen + strlen(INPUT_TAIL) + 2) ;
+  nt->inputFilename = MALLOC (pLen + dLen + strlen(INPUT_TAIL) + 2) ;
+  ASSERT (nt->inputFilename != NULL) ;
   sprintf (nt->inputFilename,"%s/%s%s",tapeDirectory,peerName,INPUT_TAIL) ;
 
-  nt->outputFilename = xmalloc (pLen + dLen + strlen(OUTPUT_TAIL) + 2) ;
+  nt->outputFilename = MALLOC (pLen + dLen + strlen(OUTPUT_TAIL) + 2) ;
+  ASSERT (nt->outputFilename != NULL) ;
   sprintf (nt->outputFilename,"%s/%s%s",tapeDirectory,peerName,OUTPUT_TAIL) ;
 
   if ( !lockFile (nt->lockFilename) )
     {
-      warn ("ME lock failed for host: %s", nt->lockFilename) ;
+      syslog (LOG_ERR,NO_LOCK_TAPE,nt->lockFilename) ;
       
-      free (nt->handFilename) ;
-      free (nt->lockFilename) ;
-      free (nt->inputFilename) ;
-      free (nt->outputFilename) ;
-      free (nt) ;
+      FREE (nt->handFilename) ;
+      FREE (nt->lockFilename) ;
+      FREE (nt->inputFilename) ;
+      FREE (nt->outputFilename) ;
+      FREE (nt) ;
 
       return NULL ;
     }
@@ -390,9 +418,9 @@ static void initTape (Tape nt)
   nt->lossage = 0 ;
 
   nt->noBacklog = false ;
-  nt->outputLowLimit = BLOGLIMIT;
-  nt->outputHighLimit = BLOGLIMIT_HIGH;
-  nt->backlogFactor = LIMIT_FUDGE;
+  nt->backlogFactor = 0.0 ;
+  nt->outputLowLimit = 0 ;
+  nt->outputHighLimit = 0 ;
 
   if (!talkToSelf)
     {
@@ -410,18 +438,17 @@ static void initTape (Tape nt)
               if (!getInteger (s,"backlog-limit-highwater",
                                &nt->outputHighLimit,INHERIT))
                 {
-                  warn ("%s no backlog-factor or backlog-limit-highwater",
-                        nt->peerName) ;
-                  nt->outputLowLimit = BLOGLIMIT;
-                  nt->outputHighLimit = BLOGLIMIT_HIGH;
-                  nt->backlogFactor = LIMIT_FUDGE;
+                  syslog (LOG_WARNING,NO_FACTOR,nt->peerName) ;
+                  nt->outputLowLimit = 0 ;
+                  nt->outputHighLimit = 0 ;
+                  nt->backlogFactor = 0.0 ;
                 }
             }
           else
             nt->outputHighLimit = (long)(nt->outputLowLimit * nt->backlogFactor);
         }
       else
-        warn ("ME config: no definition for required key backlog-limit") ;
+        syslog (LOG_ERR,NODEFN,"backlog-limit") ;
     }
   
   d_printf (1, "%s spooling: %s\n", nt->peerName, 
@@ -437,9 +464,9 @@ static void initTape (Tape nt)
 
 void gFlushTapes (void)
 {
-  unsigned int i ;
+  u_int i ;
 
-  notice ("ME flushing tapes") ;
+  syslog (LOG_NOTICE,FLUSHING_TAPES) ;
   for (i = 0 ; i < activeTapeIdx ; i++)
     tapeFlush (activeTapes [i]) ;
 }
@@ -464,17 +491,17 @@ void tapeFlush (Tape tape)
 
 
 
-void gPrintTapeInfo (FILE *fp, unsigned int indentAmt)
+void gPrintTapeInfo (FILE *fp, u_int indentAmt)
 {
   char indent [INDENT_BUFFER_SIZE] ;
-  unsigned int i ;
+  u_int i ;
   
   for (i = 0 ; i < MIN(INDENT_BUFFER_SIZE - 1,indentAmt) ; i++)
     indent [i] = ' ' ;
   indent [i] = '\0' ;
 
-  fprintf (fp,"%sGlobal Tape List : (count %lu) {\n",
-           indent,(unsigned long) activeTapeIdx) ;
+  fprintf (fp,"%sGlobal Tape List : (count %d) {\n",
+           indent,activeTapeIdx) ;
 
   for (i = 0 ; i < activeTapeIdx ; i++)
     printTapeInfo (activeTapes [i],fp,indentAmt + INDENT_INCR) ;
@@ -485,13 +512,11 @@ void gPrintTapeInfo (FILE *fp, unsigned int indentAmt)
 void tapeLogGlobalStatus (FILE *fp)
 {
   fprintf (fp,"%sBacklog file global values:%s\n",
-           genHtml ? "<B>" : "", genHtml ? "</B>" : "") ;
+           genHtml ? "<H2>" : "", genHtml ? "</H2>" : "") ;
   fprintf (fp,"        directory: %s\n",tapeDirectory) ;
   fprintf (fp,"    rotate period: %-3ld seconds\n",(long) rotatePeriod) ;
   fprintf (fp,"checkpoint period: %-3ld seconds\n",(long) tapeCkPtPeriod) ;
   fprintf (fp,"   newfile period: %-3ld seconds\n",(long) tapeCkNewFilePeriod);
-  fprintf (fp,"backlog highwater: %u\n", tapeHighwater);
-  fprintf (fp,"  highwater queue: %u\n", hostHighwater);
   fprintf (fp,"\n") ;
 }
 
@@ -518,10 +543,10 @@ void tapeLogStatus (Tape tape, FILE *fp)
     }
 }
 
-void printTapeInfo (Tape tape, FILE *fp, unsigned int indentAmt)
+void printTapeInfo (Tape tape, FILE *fp, u_int indentAmt)
 {
   char indent [INDENT_BUFFER_SIZE] ;
-  unsigned int i ;
+  u_int i ;
 #if 0
   QueueElem qe ;
 #endif
@@ -530,7 +555,7 @@ void printTapeInfo (Tape tape, FILE *fp, unsigned int indentAmt)
     indent [i] = ' ' ;
   indent [i] = '\0' ;
 
-  fprintf (fp,"%sTape : %p {\n",indent,(void *) tape) ;
+  fprintf (fp,"%sTape : %p {\n",indent,tape) ;
 
   if (tape == NULL)
     {
@@ -543,8 +568,8 @@ void printTapeInfo (Tape tape, FILE *fp, unsigned int indentAmt)
   fprintf (fp,"%s    output-file : %s\n",indent, tape->outputFilename) ;
   fprintf (fp,"%s    lock-file : %s\n",indent, tape->lockFilename) ;
   fprintf (fp,"%s    peerName : %s\n",indent,tape->peerName) ;
-  fprintf (fp,"%s    input-FILE : %p\n",indent, (void *) tape->inFp) ;
-  fprintf (fp,"%s    output-FILE : %p\n",indent, (void *) tape->outFp) ;
+  fprintf (fp,"%s    input-FILE : %p\n",indent, tape->inFp) ;
+  fprintf (fp,"%s    output-FILE : %p\n",indent, tape->outFp) ;
   fprintf (fp,"%s    output-limit : %ld\n",indent, tape->outputLowLimit) ;
 
 #if 0
@@ -586,12 +611,12 @@ void delTape (Tape tape)
 #if 1
   
   if (tape->outFp != NULL && fclose (tape->outFp) != 0)
-    syswarn ("ME ioerr fclose %s", tape->outputFilename) ;
+    syslog (LOG_ERR,FCLOSE_FAILED, tape->outputFilename) ;
 
   if (stat(tape->outputFilename, &st) == 0 && st.st_size == 0)
     {
       d_printf (1,"removing empty output tape: %s\n",tape->outputFilename) ;
-      unlink (tape->outputFilename) ;
+      (void) unlink (tape->outputFilename) ;
     }
   
   tape->outFp = NULL ;
@@ -606,7 +631,7 @@ void delTape (Tape tape)
   if (tape->inFp != NULL)
     {
       checkpointTape (tape) ;
-      fclose (tape->inFp) ;
+      (void) fclose (tape->inFp) ;
     }
 
   unlockFile (tape->lockFilename) ;
@@ -619,7 +644,7 @@ void delTape (Tape tape)
              
   removeTapeGlobally (tape) ;
   
-  free (tape) ;
+  FREE (tape) ;
 }
 
 
@@ -628,10 +653,10 @@ void tapeTakeArticle (Tape tape, Article article)
 #if 0
   QueueElem elem ;
 #endif
+  int amt ;
   const char *fname, *msgid ;
 
   ASSERT (tape != NULL) ;
-  ASSERT (article != NULL) ;
 
   /* return immediately if spooling disabled - jgarzik */
   if (tape->noBacklog)
@@ -642,14 +667,25 @@ void tapeTakeArticle (Tape tape, Article article)
 
   fname = artFileName (article) ;
   msgid = artMsgId (article) ;
-  fprintf (tape->outFp,"%s %s\n", fname, msgid);
+  amt = fprintf (tape->outFp,"%s %s\n", fname, msgid) ;
+
   /* I'd rather know where I am each time, and I don't trust all
-   * fprintf's to give me character counts.  Therefore, do not use:
-   *   tape->outputSize += (return value of the previous fprintf call);
-   * nor:
-   *   tape->outputSize = ftello (tape->outFp);
-   */
+    fprintf's to give me character counts. */
+#if defined (TRUST_FPRINTF)
+
+  tape->outputSize += amt ;
+
+#else
+#if defined (NO_TRUST_STRLEN)
+
+  tape->outputSize = ftell (tape->outFp) ;
+
+#else
+
   tape->outputSize += strlen(fname) + strlen(msgid) + 2 ; /* " " + "\n" */
+
+#endif
+#endif
   
   delArticle (article) ;
 
@@ -660,17 +696,18 @@ void tapeTakeArticle (Tape tape, Article article)
       fflush (tape->outFp) ;
       
       if (fstat (fileno (tape->outFp),&sb) != 0)
-        syswarn ("ME oserr fstat %s",tape->outputFilename) ;
+        syslog (LOG_ERR,FSTAT_FAILURE,tape->outputFilename) ;
       else if (sb.st_size != tape->outputSize) 
-        syslog (LOG_ERR,"fstat and ftello do not agree: %ld %ld for %s\n",
+        syslog (LOG_ERR,"fstat and ftell do not agree: %ld %ld for %s\n",
                 (long)sb.st_size,tape->outputSize,tape->outputFilename) ;
     }
   
   if (tape->outputHighLimit > 0 && tape->outputSize >= tape->outputHighLimit)
     {
       long oldSize = tape->outputSize ;
-      shrinkfile (tape->outFp,tape->outputLowLimit,tape->outputFilename,"a+");
-      tape->outputSize = ftello (tape->outFp) ;
+      (void) shrinkfile (tape->outFp,
+                         tape->outputLowLimit,tape->outputFilename,"a+") ;
+      tape->outputSize = ftell (tape->outFp) ;
       tape->lossage += oldSize - tape->outputSize ;
     }
 }
@@ -698,19 +735,19 @@ Article getArticle (Tape tape)
       if (fgets (line,sizeof (line), tape->inFp) == NULL)
         {
           if (ferror (tape->inFp))
-            syswarn ("ME ioerr on tape file %s", tape->inputFilename) ;
+            syslog (LOG_ERR,TAPE_INPUT_ERROR,tape->inputFilename) ;
           else if ( !feof (tape->inFp) )
-            syswarn ("ME oserr fgets %s", tape->inputFilename) ;
+            syslog (LOG_ERR,FGETS_FAILED,tape->inputFilename) ;
           
           if (fclose (tape->inFp) != 0)
-            syswarn ("ME ioerr fclose %s", tape->inputFilename) ;
+            syslog (LOG_ERR,FCLOSE_FAILED, tape->inputFilename);
 
           d_printf (1,"No more articles on tape %s\n",tape->inputFilename) ;
 
           tape->inFp = NULL ;
           tape->scribbled = false ;
 
-          unlink (tape->inputFilename) ;
+          (void) unlink (tape->inputFilename) ;
 
           if ((now - tape->lastRotated) > rotatePeriod)
             prepareFiles (tape) ; /* rotate files to try next. */
@@ -719,7 +756,7 @@ Article getArticle (Tape tape)
         {
           msgid = filename = NULL ;
           
-          for (p = line ; *p && isspace((unsigned char) *p) ; p++) /* eat whitespace */
+          for (p = line ; *p && isspace (*p) ; p++) /* eat whitespace */
             /* nada */ ;
 
           if (*p != '\0')
@@ -731,7 +768,7 @@ Article getArticle (Tape tape)
                   filename = p ;
                   *q = '\0' ;
       
-                  for (q++ ; *q && isspace((unsigned char) *q) ; q++)
+                  for (q++ ; *q && isspace (*q) ; q++) /* eat more white */
                     /* nada */ ;
 
                   if (*q != '\0')
@@ -758,8 +795,7 @@ Article getArticle (Tape tape)
               ;
             if (p > msgid) p--;
             if (*msgid != '<' || *p != '>') {
-              warn ("ME tape invalid messageID in %s: %s",
-                    tape->inputFilename,msgid);
+              syslog (LOG_ERR,TAPE_SPAGHETTI,tape->inputFilename,msgid);
               msgid = NULL;
             }
           }
@@ -781,13 +817,13 @@ Article getArticle (Tape tape)
         {
           /* last article read was the end of the tape. */
           if (fclose (tape->inFp) != 0)
-            syswarn ("ME ioerr fclose %s", tape->inputFilename) ;
+            syslog (LOG_ERR,FCLOSE_FAILED,tape->inputFilename) ;
           
           tape->inFp = NULL ;
           tape->scribbled = false ;
           
           /* toss out the old input file and prepare the new one */
-          unlink (tape->inputFilename) ;
+          (void) unlink (tape->inputFilename) ;
           
           if (now - tape->lastRotated > rotatePeriod)
             prepareFiles (tape) ;
@@ -812,7 +848,7 @@ Article getArticle (Tape tape)
 /* Cause all the Tapes to checkpoint themselves. */
 void checkPointTapes (void)
 {
-  unsigned int i ;
+  u_int i ;
 
   for (i = 0 ; i < activeTapeIdx ; i++)
     checkpointTape (activeTapes [i]) ;
@@ -822,7 +858,7 @@ void checkPointTapes (void)
 /* make all the tapes set their checkNew flag. */
 static void tapesSetCheckNew (void) 
 {
-  unsigned int i ;
+  u_int i ;
 
   for (i = 0 ; i < activeTapeIdx ; i++)
     activeTapes[i]->checkNew = true ;
@@ -844,7 +880,7 @@ void setTapeDirectory (const char *newDir)
   if (tapeDirectory != NULL)
     freeCharP (tapeDirectory) ;
   
-  tapeDirectory = xstrdup (newDir) ;
+  tapeDirectory = strdup (newDir) ;
 
   addPointerFreedOnExit (tapeDirectory) ;
 }
@@ -858,7 +894,7 @@ const char *getTapeDirectory (void)
 #if 0
   if (tapeDirectory == NULL)
     {
-      tapeDirectory = xstrdup (dflTapeDir) ;
+      tapeDirectory = strdup (dflTapeDir) ;
       addPointerFreedOnExit (tapeDirectory) ;
     }
 #endif
@@ -891,13 +927,15 @@ static void addTapeGlobally (Tape tape)
   
   if (activeTapeSize == activeTapeIdx)
     {
-      unsigned int i ;
+      u_int i ;
       
       activeTapeSize += 10 ;
       if (activeTapes != NULL)
-        activeTapes = xrealloc (activeTapes, sizeof(Tape) * activeTapeSize) ;
+        activeTapes = REALLOC (activeTapes, Tape, activeTapeSize) ;
       else
-        activeTapes = xmalloc (sizeof(Tape) * activeTapeSize) ;
+        activeTapes = ALLOC (Tape, activeTapeSize) ;
+
+      ASSERT (activeTapes != NULL) ;
 
       for (i = activeTapeIdx ; i < activeTapeSize ; i++)
         activeTapes [i] = NULL ;
@@ -909,7 +947,7 @@ static void addTapeGlobally (Tape tape)
 /* Remove a tape for the class-level list of active tapes. */
 static void removeTapeGlobally (Tape tape)
 {
-  unsigned int i ;
+  u_int i ;
 
   if (tape == NULL)
     return ;
@@ -929,7 +967,7 @@ static void removeTapeGlobally (Tape tape)
 
   if (activeTapeIdx == 0)
     {
-      free (activeTapes) ;
+      FREE (activeTapes) ;
       activeTapes = NULL ;
     }
 }
@@ -948,9 +986,10 @@ static void checkpointTape (Tape tape)
       return ;
     }
   
-  if ((tape->tellpos = ftello (tape->inFp)) < 0)
+  if ((tape->tellpos = ftell (tape->inFp)) < 0)
     {
-      syswarn ("ME oserr ftello %s", tape->inputFilename) ;
+      syslog (LOG_ERR,FTELL_FAILED,tape->inputFilename) ;
+
       return ;
     }
 
@@ -987,9 +1026,8 @@ static void checkpointTape (Tape tape)
       
       fprintf (tape->inFp,"%ld",tape->tellpos) ;
 
-      if (fseeko (tape->inFp,tape->tellpos,SEEK_SET) != 0)
-        syswarn ("ME oserr fseeko(%s,%ld,SEEK_SET)",tape->inputFilename,
-                 tape->tellpos) ;
+      if (fseek (tape->inFp,tape->tellpos,SEEK_SET) != 0)
+        syslog (LOG_ERR,FSEEK_FAILED,tape->inputFilename,tape->tellpos) ;
     }
   
   tape->changed = false ;
@@ -1039,7 +1077,7 @@ static void prepareFiles (Tape tape)
     {
       newExists = fileExistsP (tape->handFilename) ;
       if (newExists)
-        notice ("%s new hand-prepared backlog file", tape->peerName) ;
+        syslog (LOG_NOTICE,NEW_HAND_FILE,tape->peerName) ;
     }
   else
     newExists = false ;
@@ -1060,11 +1098,10 @@ static void prepareFiles (Tape tape)
   if (newExists && !inpExists)
     {
       if (rename (tape->handFilename,tape->inputFilename) != 0)
-        syswarn ("ME oserr rename %s, %s", tape->handFilename,
-                 tape->inputFilename);
+        syslog (LOG_ERR,RENAME_FAILED,tape->handFilename, tape->inputFilename);
       else
         {
-          notice ("%s grabbing external tape file", tape->peerName) ;
+          syslog (LOG_NOTICE,ROTATING_HAND_DROPPED,tape->peerName) ;
           inpExists = true ;
         }
     }
@@ -1080,10 +1117,12 @@ static void prepareFiles (Tape tape)
         }
       
       if (rename (tape->outputFilename,tape->inputFilename) != 0)
-        syswarn ("ME oserr rename %s, %s", tape->outputFilename,
-                 tape->inputFilename) ;
+        syslog (LOG_ERR,RENAME_FAILED,
+                tape->outputFilename,tape->inputFilename);
       else
         inpExists = true ;
+
+      outExists = false ;
     }
 
   /* now open up the input file and seek to the proper position. */
@@ -1093,7 +1132,7 @@ static void prepareFiles (Tape tape)
       long flength ;
       
       if ((tape->inFp = fopen (tape->inputFilename,"r+")) == NULL)
-        syswarn ("ME fopen %s", tape->inputFilename) ;
+        syslog (LOG_ERR,FOPEN_FAILURE,tape->inputFilename) ;
       else
         {
           char buffer [64] ;
@@ -1106,7 +1145,7 @@ static void prepareFiles (Tape tape)
                   unlink (tape->inputFilename) ;
                 }
               else
-                syswarn ("ME oserr fgets %s", tape->inputFilename) ;
+                syslog (LOG_ERR,FGETS_FAILED,tape->inputFilename) ;
               
               fclose (tape->inFp) ;
               tape->inFp = NULL ;
@@ -1114,7 +1153,7 @@ static void prepareFiles (Tape tape)
             }
           else
             {
-              unsigned int len = strlen (buffer) ;
+              u_int len = strlen (buffer) ;
               long newPos = 0 ;
 
               if (len > 0 && buffer [len - 1] == '\n')
@@ -1131,15 +1170,14 @@ static void prepareFiles (Tape tape)
 
               if ((flength = fileLength (fileno (tape->inFp))) < tape->tellpos)
                 {
-                  warn ("ME tape short: %s %ld %ld", tape->inputFilename,
-                        flength, tape->tellpos) ;
+                  syslog (LOG_ERR,FILE_SHORT,tape->inputFilename,
+                          flength, tape->tellpos);
                   tape->tellpos = 0 ;
                 }
               else if (tape->tellpos == 0)
                 rewind (tape->inFp) ;
-              else if (fseeko (tape->inFp,tape->tellpos - 1,SEEK_SET) != 0)
-                syswarn ("ME oserr fseeko(%s,%ld,SEEK_SET)",
-                         tape->inputFilename,tape->tellpos) ;
+              else if (fseek (tape->inFp,tape->tellpos - 1,SEEK_SET) != 0)
+                syslog (LOG_ERR,FSEEK_FAILED,tape->inputFilename,tape->tellpos);
               else if ((c = fgetc (tape->inFp)) != '\n')
                 {
                   while (c != EOF && c != '\n')
@@ -1159,9 +1197,8 @@ static void prepareFiles (Tape tape)
                       
                       tape->changed = true ;
                       checkpointTape (tape) ;
-
-                      warn ("ME internal checkpoint line boundary missed:"
-                            " %s %ld vs. %ld",tape->inputFilename,
+                      
+                      syslog (LOG_ERR,CKPT_BNDRY,tape->inputFilename,
                               tape->tellpos, oldPos) ;
                     }
                 }
@@ -1177,18 +1214,20 @@ static void prepareFiles (Tape tape)
     {
       if ((tape->outFp = fopen (tape->outputFilename,"a+")) == NULL)
         {
-          syswarn ("ME tape open failed (a+) %s", tape->outputFilename) ;
+          syslog (LOG_ERR,TAPE_OPEN_FAILED, "a+", tape->outputFilename) ;
           return ;
         }
-      fseeko (tape->outFp,0,SEEK_END) ;
-      tape->outputSize = ftello (tape->outFp) ;
+      fseek (tape->outFp,0,SEEK_END) ;
+      tape->outputSize = ftell (tape->outFp) ;
       tape->lossage = 0 ;
     }
 }
 
 
-static void tapeCkNewFileCbk (TimeoutId id, void *d UNUSED)
+static void tapeCkNewFileCbk (TimeoutId id, void *d)
 {
+  (void) d ;
+  
   ASSERT (id == ckNewFileId) ;
   ASSERT (tapeCkNewFilePeriod > 0) ;
 
@@ -1199,8 +1238,10 @@ static void tapeCkNewFileCbk (TimeoutId id, void *d UNUSED)
 
 
 /* The timer callback function that will checkpoint all the active tapes. */
-static void tapeCheckpointCallback (TimeoutId id, void *d UNUSED)
+static void tapeCheckpointCallback (TimeoutId id, void *d)
 {
+  (void) d ;                    /* keep lint happy */
+
   ASSERT (id == checkPtId) ;
   ASSERT (tapeCkPtPeriod > 0) ;
   
@@ -1228,7 +1269,7 @@ static void flushTape (Tape tape)
       
       delArticle (elem->article) ;
       
-      free (elem) ;
+      FREE (elem) ;
       elem = tape->head ;
     }
   tape->tail = NULL ;
@@ -1237,7 +1278,7 @@ static void flushTape (Tape tape)
 
   /* I'd rather know where I am each time, and I don't trust all
      fprintf's to give me character counts. */
-  tape->outputSize = ftello (tape->outFp) ;
+  tape->outputSize = ftell (tape->outFp) ;
   if (debugShrinking)
     {
       struct stat buf ;
@@ -1251,16 +1292,16 @@ static void flushTape (Tape tape)
         }
       else if (buf.st_size != tape->outputSize)
         {
-          warn ("ME fstat and ftello do not agree for %s",
-                tape->outputFilename) ;
+          syslog (LOG_ERR,FSTAT_NE_FTELL,tape->outputFilename) ;
           logged = true ;
         }
     }
   
   if (tape->outputHighLimit > 0 && tape->outputSize > tape->outputHighLimit)
     {
-      shrinkfile (tape->outFp,tape->outputLowLimit,tape->outputFilename,"a+");
-      tape->outputSize = ftello (tape->outFp) ;
+      (void) shrinkfile (tape->outFp,
+                         tape->outputLowLimit,tape->outputFilename,"a+") ;
+      tape->outputSize = ftell (tape->outFp) ;
     }
 }
 #endif
@@ -1268,6 +1309,5 @@ static void flushTape (Tape tape)
 
 static void tapeCleanup (void)
 {
-  free (tapeDirectory) ;
-  tapeDirectory = NULL ;
+  FREE (tapeDirectory) ;
 }
