@@ -1,15 +1,14 @@
-/*  $Revision: 1.48.2.7 $
+/*  $Id: getconfig.c,v 1.71.2.3 2000/07/17 06:46:14 rra Exp $
 **
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include "configdata.h"
+#include "config.h"
 #include "clibrary.h"
-#include "paths.h"
+#include <syslog.h> 
+
+#include "innconf.h"
 #include "libinn.h"
 #include "macros.h"
-#include <syslog.h> 
-#include "innconf.h"
+#include "paths.h"
 
 /* Global and initialized; to work around SunOS -Bstatic bug, sigh. */
 STATIC char		ConfigBuff[SMBUF] = "";
@@ -35,9 +34,9 @@ STATIC int		ConfigBitsize;
 	include/innconf.h	:	Add to conf_defaults
 	include/libinn.h	:	Add varname to conf_vars struct
 	lib/getconfig.c		:	SetDefaults() & ReadInnConf(), ClearInnConf()
-	samples/inn.conf	:	Set the default value
-	samples/inncheck.in	:	So we can check it
-	doc/inn.conf.5		:	Document it!
+	samples/inn.conf.in	:	Set the default value
+	scripts/inncheck.in	:	So we can check it
+	doc/pod/inn.conf.pod	:	Document it!
 	wherever you need it	:	Use as innconf->varname
 */
 
@@ -116,11 +115,7 @@ char *GetConfigValue(char *value)
 
     /* Some values have defaults if not in the file. */
     if (EQ(value, _CONF_FROMHOST) || EQ(value, _CONF_PATHHOST))
-	return GetFQDN();
-    if (EQ(value, _CONF_MIMECONTENTTYPE))
-	return "text/plain; charset=US-ASCII";
-    if (EQ(value, _CONF_MIMEENCODING))
-	return "7bit";
+	return GetFQDN(innconf->domain);
     return NULL;
 }
 
@@ -170,20 +165,14 @@ void SetDefaults()
     }
     innconf->moderatormailer = NULL;
     innconf->domain = NULL;
-    innconf->mimeversion = NULL;
-    innconf->mimecontenttype = NULL;
-    innconf->mimeencoding = NULL;
     innconf->hiscachesize = 0;
-    innconf->wireformat = FALSE;
     innconf->xrefslave = FALSE;
     innconf->complaints = NULL;
     innconf->spoolfirst = FALSE;
-    innconf->writelinks = TRUE;
+    innconf->immediatecancel = FALSE;
     innconf->timer = 0;
     innconf->status = 0;
-    innconf->storageapi = FALSE;
     innconf->articlemmap = FALSE;
-    innconf->overviewmmap = TRUE;
     innconf->mta = NULL;
     innconf->mailcmd = NULL;
     innconf->checkincludedtext = FALSE;
@@ -205,7 +194,7 @@ void SetDefaults()
     innconf->chaninacttime = 600;
     innconf->maxconnections = 50;
     innconf->chanretrytime = 300;
-    innconf->artcutoff = 14 * 24 * 60 * 60;
+    innconf->artcutoff = 10 * 24 * 60 * 60;
     innconf->pauseretrytime = 300;
     innconf->nntplinklog = FALSE;
     innconf->nntpactsync = 200;
@@ -221,7 +210,6 @@ void SetDefaults()
     innconf->port = NNTP_PORT;
     innconf->readertrack = FALSE;
     innconf->strippostcc = FALSE;
-    innconf->overviewname = NULL;
     innconf->keywords = FALSE;		
     innconf->keylimit = 512 ;		
     innconf->keyartlimit = 100000;
@@ -229,6 +217,9 @@ void SetDefaults()
     innconf->nnrpdposthost = NULL;
     innconf->nnrpdpostport = NNTP_PORT;
     innconf->nnrpperlauth = FALSE;
+    innconf->nnrppythonauth = FALSE;
+    innconf->addnntppostinghost = TRUE;
+    innconf->addnntppostingdate = TRUE;
 
     innconf->pathnews = NULL;
     innconf->pathbin = NULL;
@@ -246,13 +237,10 @@ void SetDefaults()
     innconf->pathincoming = NULL;
     innconf->patharchive = NULL;
     innconf->pathtmp = NULL;
-    innconf->pathuniover = NULL;
 
     innconf->logsitename = TRUE;
-    innconf->extendeddbz = FALSE;
     innconf->nnrpdoverstats = FALSE;
     innconf->storeonxref = TRUE;
-    innconf->decnetdomain = NULL;
     innconf->backoff_auth = FALSE;
     innconf->backoff_db = NULL;
     innconf->backoff_k = 1L;
@@ -261,14 +249,24 @@ void SetDefaults()
     innconf->backoff_trigger = 10000L;
     innconf->refusecybercancels = FALSE;
     innconf->nnrpdcheckart = TRUE;
-    innconf->activedenable = FALSE;
-    innconf->activedupdate = 30L;
-    innconf->activedport = 1119;
-    innconf->storemsgid = TRUE;
     innconf->nicenewnews = 0;
+    innconf->nicennrpd = 0;
     innconf->usecontrolchan = FALSE;
     innconf->mergetogroups = FALSE;
     innconf->noreader = FALSE;
+    innconf->nnrpdauthsender = FALSE;
+    innconf->cnfscheckfudgesize = 0L;
+    innconf->rlimitnofile = -1;
+    innconf->ignorenewsgroups = FALSE;
+    innconf->overcachesize = 15;
+    innconf->enableoverview = TRUE;
+    innconf->wireformat = FALSE;
+    innconf->ovmethod = NULL;
+    innconf->useoverchan = FALSE;
+    innconf->ovgrouppat = NULL;
+    innconf->groupbaseexpiry = TRUE;
+    innconf->wipcheck = 5;
+    innconf->wipexpire = 10;
 }
 
 void ClearInnConf()
@@ -280,15 +278,11 @@ void ClearInnConf()
     if (innconf->organization != NULL) DISPOSE(innconf->organization);
     if (innconf->moderatormailer != NULL) DISPOSE(innconf->moderatormailer);
     if (innconf->domain != NULL) DISPOSE(innconf->domain);
-    if (innconf->mimeversion != NULL) DISPOSE(innconf->mimeversion);
-    if (innconf->mimecontenttype != NULL) DISPOSE(innconf->mimecontenttype);
-    if (innconf->mimeencoding != NULL) DISPOSE(innconf->mimeencoding);
     if (innconf->complaints != NULL) DISPOSE(innconf->complaints);
     if (innconf->mta != NULL) DISPOSE(innconf->mta);
     if (innconf->mailcmd != NULL) DISPOSE(innconf->mailcmd);
     if (innconf->bindaddress != NULL) DISPOSE(innconf->bindaddress);
     if (innconf->sourceaddress != NULL) DISPOSE(innconf->sourceaddress);
-    if (innconf->overviewname != NULL) DISPOSE(innconf->overviewname);
     if (innconf->nnrpdposthost != NULL) DISPOSE(innconf->nnrpdposthost);
 
     if (innconf->pathnews != NULL) DISPOSE(innconf->pathnews);
@@ -307,9 +301,9 @@ void ClearInnConf()
     if (innconf->pathincoming != NULL) DISPOSE(innconf->pathincoming);
     if (innconf->patharchive != NULL) DISPOSE(innconf->patharchive);
     if (innconf->pathtmp != NULL) DISPOSE(innconf->pathtmp);
-    if (innconf->pathuniover != NULL) DISPOSE(innconf->pathuniover);
-    if (innconf->decnetdomain != NULL) DISPOSE(innconf->decnetdomain);
     if (innconf->backoff_db != NULL) DISPOSE(innconf->backoff_db);
+    if (innconf->ovmethod != NULL) DISPOSE(innconf->ovmethod);
+    if (innconf->ovgrouppat != NULL) DISPOSE(innconf->ovgrouppat);
     memset(ConfigBit, '\0', ConfigBitsize);
 }
 
@@ -320,17 +314,17 @@ void ClearInnConf()
 int CheckInnConf()
 {
     static char *tmpdir = NULL;
-    static int dirlen = 0;
+    static unsigned int dirlen = 0;
 
-    if (GetFQDN() == NULL) {
+    if (GetFQDN(innconf->domain) == NULL) {
 	syslog(L_FATAL, "Hostname does not resolve or 'domain' in inn.conf is missing");
 	(void)fprintf(stderr, "Hostname does not resolve or 'domain' in inn.conf is missing");
 	return(-1);
     }
     if (innconf->fromhost == NULL) {
-	innconf->fromhost = COPY(GetFQDN());
+	innconf->fromhost = COPY(GetFQDN(innconf->domain));
     }
-    if (innconf->pathhost == NULL && ((innconf->pathhost = COPY(GetFQDN())) == NULL)) {
+    if (innconf->pathhost == NULL && ((innconf->pathhost = COPY(GetFQDN(innconf->domain))) == NULL)) {
 	syslog(L_FATAL, "Must set 'pathhost' in inn.conf");
 	(void)fprintf(stderr, "Must set 'pathhost' in inn.conf");
 	return(-1);
@@ -340,12 +334,6 @@ int CheckInnConf()
 	(void)fprintf(stderr, "Must set 'mta' in inn.conf");
 	return(-1);
     }
-    if (innconf->overviewname == NULL) 
-	innconf->overviewname = COPY(".overview");
-
-    if (innconf->storageapi != TRUE)
-	innconf->extendeddbz = FALSE;
-
     if (innconf->pathnews == NULL) {
 	syslog(L_FATAL, "Must set 'pathnews' in inn.conf");
 	(void)fprintf(stderr, "Must set 'pathnews' in inn.conf");
@@ -399,9 +387,6 @@ int CheckInnConf()
     if (innconf->mailcmd == NULL) {
 	innconf->mailcmd = COPY(cpcatpath(innconf->pathbin, "innmail"));
     }
-    if (innconf->pathuniover == NULL) {
-	innconf->pathuniover = COPY(cpcatpath(innconf->pathspool, "uniover"));
-    }
     /* Set the TMPDIR variable unconditionally and globally */
     if (8 + strlen(innconf->pathtmp) > dirlen)
 	dirlen = 8 + strlen(innconf->pathtmp);
@@ -412,6 +397,11 @@ int CheckInnConf()
     sprintf(tmpdir, "TMPDIR=%s", innconf->pathtmp);
     putenv(tmpdir);
     /* tmpdir should not be freed for some OS */
+    if (innconf->enableoverview && innconf->ovmethod == NULL) {
+	syslog(L_FATAL, "'ovmethod' must be defined in inn.conf if enableoverview is true");
+	(void)fprintf(stderr, "'ovmethod' must be defined in inn.conf if enableoverview is true\n");
+	return(-1);
+    }
 
     return(0);
 }
@@ -496,33 +486,11 @@ int ReadInnConf()
 		if (!bit) innconf->domain = COPY(p);
 		SET_CONFIG(CONF_VAR_DOMAIN);
 	    } else
-	    if (EQ(ConfigBuff,_CONF_MIMEVERSION)) {
-		TEST_CONFIG(CONF_VAR_MIMEVERSION, bit);
-		if (!bit) innconf->mimeversion = COPY(p);
-		SET_CONFIG(CONF_VAR_MIMEVERSION);
-	    } else
-	    if (EQ(ConfigBuff,_CONF_MIMECONTENTTYPE)) {
-		TEST_CONFIG(CONF_VAR_MIMECONTENTTYPE, bit);
-		if (!bit) innconf->mimecontenttype = COPY(p);
-		SET_CONFIG(CONF_VAR_MIMECONTENTTYPE);
-	    } else
-	    if (EQ(ConfigBuff,_CONF_MIMEENCODING)) {
-		TEST_CONFIG(CONF_VAR_MIMEENCODING, bit);
-		if (!bit) innconf->mimeencoding = COPY(p);
-		SET_CONFIG(CONF_VAR_MIMEENCODING);
-	    } else
 	    if (EQ(ConfigBuff,_CONF_HISCACHESIZE)) {
 		TEST_CONFIG(CONF_VAR_HISCACHESIZE, bit);
 		if (!bit) innconf->hiscachesize = atoi(p);
 		SET_CONFIG(CONF_VAR_HISCACHESIZE);
 	    } else
-	    if (EQ(ConfigBuff,_CONF_WIREFORMAT)) {
-		TEST_CONFIG(CONF_VAR_WIREFORMAT, bit);
-		if (!bit) {
-		if (innconf->storageapi != TRUE && boolval != -1) innconf->wireformat = boolval;
-		}
-		SET_CONFIG(CONF_VAR_WIREFORMAT);
-	    } else
 	    if (EQ (ConfigBuff,_CONF_XREFSLAVE)) {
 		TEST_CONFIG(CONF_VAR_XREFSLAVE, bit);
 		if (!bit && boolval != -1) innconf->xrefslave = boolval;
@@ -538,10 +506,10 @@ int ReadInnConf()
 		if (!bit && boolval != -1) innconf->spoolfirst = boolval;
 		SET_CONFIG(CONF_VAR_SPOOLFIRST);
 	    } else
-	    if (EQ(ConfigBuff,_CONF_WRITELINKS)) {
-		TEST_CONFIG(CONF_VAR_WRITELINKS, bit);
-		if (!bit && boolval != -1) innconf->writelinks = boolval;
-		SET_CONFIG(CONF_VAR_WRITELINKS);
+	    if (EQ(ConfigBuff,_CONF_IMMEDIATECANCEL)) {
+		TEST_CONFIG(CONF_VAR_IMMEDIATECANCEL, bit);
+		if (!bit && boolval != -1) innconf->immediatecancel = boolval;
+		SET_CONFIG(CONF_VAR_IMMEDIATECANCEL);
 	    } else
 	    if (EQ(ConfigBuff,_CONF_TIMER)) {
 		TEST_CONFIG(CONF_VAR_TIMER, bit);
@@ -553,24 +521,11 @@ int ReadInnConf()
 		if (!bit) innconf->status = atoi(p);
 		SET_CONFIG(CONF_VAR_STATUS);
 	    } else
-	    if (EQ(ConfigBuff,_CONF_STORAGEAPI)) {
-		TEST_CONFIG(CONF_VAR_STORAGEAPI, bit);
-		if (!bit) {
-		if (boolval != -1) innconf->storageapi = boolval;
-		if (innconf->storageapi == TRUE) innconf->wireformat = TRUE;
-		}
-		SET_CONFIG(CONF_VAR_STORAGEAPI);
-	    } else
 	    if (EQ(ConfigBuff,_CONF_ARTICLEMMAP)) {
 		TEST_CONFIG(CONF_VAR_ARTICLEMMAP, bit);
 		if (!bit && boolval != -1) innconf->articlemmap = boolval;
 		SET_CONFIG(CONF_VAR_ARTICLEMMAP);
 	    } else
-	    if (EQ(ConfigBuff,_CONF_OVERVIEWMMAP)) {
-		TEST_CONFIG(CONF_VAR_OVERVIEWMMAP, bit);
-		if (!bit && boolval != -1) innconf->overviewmmap = boolval;
-		SET_CONFIG(CONF_VAR_OVERVIEWMMAP);
-	    } else
 	    if (EQ(ConfigBuff,_CONF_MTA)) {
 		TEST_CONFIG(CONF_VAR_MTA, bit);
 		if (!bit) innconf->mta = COPY(p);
@@ -736,16 +691,26 @@ int ReadInnConf()
 		if (!bit && boolval != -1) innconf->nnrpperlauth = boolval;
 		SET_CONFIG(CONF_VAR_NNRPPERLAUTH);
 	    } else
+	    if (EQ(ConfigBuff,_CONF_NNRPPYTHONAUTH)) {
+		TEST_CONFIG(CONF_VAR_NNRPPYTHONAUTH, bit);
+		if (!bit && boolval != -1) innconf->nnrppythonauth = boolval;
+		SET_CONFIG(CONF_VAR_NNRPPYTHONAUTH);
+	    } else
+	    if (EQ(ConfigBuff,_CONF_ADDNNTPPOSTINGHOST)) {
+		TEST_CONFIG(CONF_VAR_ADDNNTPPOSTINGHOST, bit);
+		if (!bit && boolval != -1) innconf->addnntppostinghost = boolval;
+		SET_CONFIG(CONF_VAR_ADDNNTPPOSTINGHOST);
+	    } else
+	    if (EQ(ConfigBuff,_CONF_ADDNNTPPOSTINGDATE)) {
+		TEST_CONFIG(CONF_VAR_ADDNNTPPOSTINGDATE, bit);
+		if (!bit && boolval != -1) innconf->addnntppostingdate = boolval;
+		SET_CONFIG(CONF_VAR_ADDNNTPPOSTINGDATE);
+	    } else
 	    if (EQ(ConfigBuff,_CONF_STRIPPOSTCC)) {
 		TEST_CONFIG(CONF_VAR_STRIPPOSTCC, bit);
 		if (!bit && boolval != -1) innconf->strippostcc = boolval;
 		SET_CONFIG(CONF_VAR_STRIPPOSTCC);
 	    } else
-	    if (EQ(ConfigBuff,_CONF_OVERVIEWNAME)) {
-		TEST_CONFIG(CONF_VAR_OVERVIEWNAME, bit);
-		if (!bit) innconf->overviewname = COPY(p);
-		SET_CONFIG(CONF_VAR_OVERVIEWNAME);
-	    } else
 	    if (EQ(ConfigBuff,_CONF_KEYWORDS)) {
 		TEST_CONFIG(CONF_VAR_KEYWORDS, bit);
 		if (!bit && boolval != -1) innconf->keywords = boolval;
@@ -846,11 +811,6 @@ int ReadInnConf()
 		if (!bit) innconf->pathtmp = COPY(p);
 		SET_CONFIG(CONF_VAR_PATHTMP);
 	    } else
-	    if (EQ(ConfigBuff,_CONF_PATHUNIOVER)) {
-		TEST_CONFIG(CONF_VAR_PATHUNIOVER, bit);
-		if (!bit) innconf->pathuniover = COPY(p);
-		SET_CONFIG(CONF_VAR_PATHUNIOVER);
-	    } else
 	    if (EQ(ConfigBuff,_CONF_LOGSITENAME)) {
 		TEST_CONFIG(CONF_VAR_LOGSITENAME, bit);
 		if (!bit && boolval != -1) innconf->logsitename = boolval;
@@ -866,11 +826,6 @@ int ReadInnConf()
 		if (!bit) innconf->nnrpdpostport = atol(p);
 		SET_CONFIG(CONF_VAR_NNRPDPOSTPORT);
 	    } else
-	    if (EQ(ConfigBuff,_CONF_EXTENDEDDBZ)) {
-		TEST_CONFIG(CONF_VAR_EXTENDEDDBZ, bit);
-		if (!bit && boolval != -1) innconf->extendeddbz = boolval;
-		SET_CONFIG(CONF_VAR_EXTENDEDDBZ);
-	    } else
 	    if (EQ(ConfigBuff,_CONF_NNRPDOVERSTATS)) {
 		TEST_CONFIG(CONF_VAR_NNRPDOVERSTATS, bit);
 		if (!bit && boolval != -1) innconf->nnrpdoverstats = boolval;
@@ -881,11 +836,6 @@ int ReadInnConf()
 		if (!bit && boolval != -1) innconf->storeonxref = boolval;
 		SET_CONFIG(CONF_VAR_STOREONXREF);
 	    } else
-	    if (EQ(ConfigBuff,_CONF_DECNETDOMAIN)) {
-		TEST_CONFIG(CONF_VAR_DECNETDOMAIN, bit);
-		if (!bit) innconf->decnetdomain = COPY(p);
-		SET_CONFIG(CONF_VAR_DECNETDOMAIN);
-	    } else
 	    if (EQ(ConfigBuff,_CONF_BACKOFFAUTH)) {
 		TEST_CONFIG(CONF_VAR_BACKOFFAUTH, bit);
 		if (!bit && boolval != -1) innconf->backoff_auth = boolval;
@@ -926,31 +876,16 @@ int ReadInnConf()
 		if (!bit && boolval != -1) innconf->nnrpdcheckart = boolval;
 		SET_CONFIG(CONF_VAR_NNRPDCHECKART);
 	    } else
-	    if (EQ(ConfigBuff,_CONF_ACTIVEDENABLE)) {
-		TEST_CONFIG(CONF_VAR_ACTIVEDENABLE, bit);
-		if (!bit && boolval != -1) innconf->activedenable = boolval;
-		SET_CONFIG(CONF_VAR_ACTIVEDENABLE);
-	    } else 
-	    if (EQ(ConfigBuff,_CONF_ACTIVEDUPDATE)) {
-		TEST_CONFIG(CONF_VAR_ACTIVEDUPDATE, bit);
-		if (!bit) innconf->activedupdate = atol(p);
-		SET_CONFIG(CONF_VAR_ACTIVEDUPDATE);
-	    } else 
-	    if (EQ(ConfigBuff,_CONF_ACTIVEDPORT)) {
-		TEST_CONFIG(CONF_VAR_ACTIVEDPORT, bit);
-		if (!bit) innconf->activedport = atoi(p);
-		SET_CONFIG(CONF_VAR_ACTIVEDPORT);
-	    } else 
-	    if (EQ(ConfigBuff,_CONF_STOREMSGID)) {
-		TEST_CONFIG(CONF_VAR_STOREMSGID, bit);
-		if (!bit && boolval != -1) innconf->storemsgid = boolval;
-		SET_CONFIG(CONF_VAR_STOREMSGID);
-	    } else
 	    if (EQ(ConfigBuff,_CONF_NICENEWNEWS)) {
 		TEST_CONFIG(CONF_VAR_NICENEWNEWS, bit);
 		if (!bit) innconf->nicenewnews = atoi(p);
 		SET_CONFIG(CONF_VAR_NICENEWNEWS);
 	    } else 
+	    if (EQ(ConfigBuff,_CONF_NICENNRPD)) {
+		TEST_CONFIG(CONF_VAR_NICENNRPD, bit);
+		if (!bit) innconf->nicennrpd = atoi(p);
+		SET_CONFIG(CONF_VAR_NICENNRPD);
+	    } else 
 	    if (EQ(ConfigBuff,_CONF_USECONTROLCHAN)) {
 		TEST_CONFIG(CONF_VAR_USECONTROLCHAN, bit);
 		if (!bit && boolval != -1) innconf->usecontrolchan = boolval;
@@ -975,6 +910,71 @@ int ReadInnConf()
 		    innconf->sourceaddress = COPY(p);
 		}
 		SET_CONFIG(CONF_VAR_SOURCEADDRESS);
+	    } else
+	    if (EQ(ConfigBuff,_CONF_NNRPDAUTHSENDER)) {
+		TEST_CONFIG(CONF_VAR_NNRPDAUTHSENDER, bit);
+		if (!bit && boolval != -1) innconf->nnrpdauthsender = boolval;
+		SET_CONFIG(CONF_VAR_NNRPDAUTHSENDER);
+	    } else
+	    if (EQ(ConfigBuff,_CONF_CNFSCHECKFUDGESIZE)) {
+		TEST_CONFIG(CONF_VAR_CNFSCHECKFUDGESIZE, bit);
+		if (!bit) innconf->cnfscheckfudgesize = atol(p);
+		SET_CONFIG(CONF_VAR_CNFSCHECKFUDGESIZE);
+	    } else 
+	    if (EQ(ConfigBuff,_CONF_RLIMITNOFILE)) {
+		TEST_CONFIG(CONF_VAR_RLIMITNOFILE, bit);
+		if (!bit) innconf->rlimitnofile = atoi(p);
+		SET_CONFIG(CONF_VAR_RLIMITNOFILE);
+	    } else 
+	    if (EQ(ConfigBuff,_CONF_IGNORENEWSGROUPS)) {
+		TEST_CONFIG(CONF_VAR_IGNORENEWSGROUPS, bit);
+		if (!bit && boolval != -1) innconf->ignorenewsgroups = boolval;
+		SET_CONFIG(CONF_VAR_IGNORENEWSGROUPS);
+	    } else 
+	    if (EQ(ConfigBuff,_CONF_OVERCACHESIZE)) {
+		TEST_CONFIG(CONF_VAR_OVERCACHESIZE, bit);
+		if (!bit) innconf->overcachesize = atoi(p);
+		SET_CONFIG(CONF_VAR_OVERCACHESIZE);
+	    } else 
+	    if (EQ(ConfigBuff,_CONF_ENABLEOVERVIEW)) {
+		TEST_CONFIG(CONF_VAR_ENABLEOVERVIEW, bit);
+		if (!bit && boolval != -1 ) innconf->enableoverview = boolval;
+		SET_CONFIG(CONF_VAR_ENABLEOVERVIEW);
+	    } else 
+	    if (EQ(ConfigBuff,_CONF_WIREFORMAT)) {
+		TEST_CONFIG(CONF_VAR_WIREFORMAT, bit);
+		if (!bit && boolval != -1 ) innconf->wireformat = boolval;
+		SET_CONFIG(CONF_VAR_WIREFORMAT);
+	    } else 
+	    if (EQ(ConfigBuff,_CONF_OVMETHOD)) {
+		TEST_CONFIG(CONF_VAR_OVMETHOD, bit);
+		if (!bit) innconf->ovmethod = COPY(p);
+		SET_CONFIG(CONF_VAR_OVMETHOD);
+	    } else 
+	    if (EQ(ConfigBuff,_CONF_USEOVERCHAN)) {
+		TEST_CONFIG(CONF_VAR_USEOVERCHAN, bit);
+		if (!bit && boolval != -1 ) innconf->useoverchan = boolval;
+		SET_CONFIG(CONF_VAR_USEOVERCHAN);
+	    } else 
+	    if (EQ(ConfigBuff,_CONF_OVGROUPPAT)) {
+		TEST_CONFIG(CONF_VAR_OVGROUPPAT, bit);
+		if (!bit) innconf->ovgrouppat = COPY(p);
+		SET_CONFIG(CONF_VAR_OVGROUPPAT);
+	    } else 
+	    if (EQ(ConfigBuff,_CONF_GROUPBASEEXPIRY)) {
+		TEST_CONFIG(CONF_VAR_GROUPBASEEXPIRY, bit);
+		if (!bit) innconf->groupbaseexpiry = boolval;
+		SET_CONFIG(CONF_VAR_GROUPBASEEXPIRY);
+	    } else 
+	    if (EQ(ConfigBuff,_CONF_WIPCHECK)) {
+		TEST_CONFIG(CONF_VAR_WIPCHECK, bit);
+		if (!bit) innconf->wipcheck = atoi(p);
+		SET_CONFIG(CONF_VAR_WIPCHECK);
+	    } else 
+	    if (EQ(ConfigBuff,_CONF_WIPEXPIRE)) {
+		TEST_CONFIG(CONF_VAR_WIPEXPIRE, bit);
+		if (!bit) innconf->wipexpire = atoi(p);
+		SET_CONFIG(CONF_VAR_WIPEXPIRE);
 	    }
 	}
 	(void)Fclose(F);
