@@ -1,12 +1,12 @@
-/*  $Id: xfopena.c 5381 2002-03-31 22:35:47Z rra $
+/*  $Revision: 1.2 $
 **
 */
-
-#include "config.h"
+#include <stdio.h>
+#include <sys/types.h>
+#include "configdata.h"
 #include "clibrary.h"
 #include <fcntl.h>
 
-#include "libinn.h"
 
 /*
 **  Open a file in append mode.  Since not all fopen's set the O_APPEND
