@@ -1,4 +1,4 @@
-/*  $Id: ov3.c,v 1.20.2.8 2001/02/23 07:54:36 kondou Exp $
+/*  $Id: ov3.c,v 1.20.2.5 2000/07/09 16:15:47 kondou Exp $
 **
 **  Indexed overview method.
 */
@@ -809,15 +809,10 @@ void *tradindexed_opensearch(char *group, int low, int high) {
 	base = ge->base;
     } while (ge->indexinode != oldinode);
 
-    /* Adjust the searching range if low < base since the active file can
-       be out of sync with the overview database and this is where the
-       initial lowmark comes from. */
-    if (high < base) {
-        OV3closegroup(gh, FALSE);
-        return NULL;
+    if (high < base || low < base) { /* return NULL if searching range is out */
+      OV3closegroup(gh, FALSE);
+      return NULL;
     }
-    if (low < base)
-       low = base;
 
     if (!OV3mmapgroup(gh)) {
 	OV3closegroup(gh, FALSE);
@@ -997,7 +992,7 @@ STATIC BOOL OV3packgroup(char *group, int delta) {
     }
 
     /* stat old index file so we know its actual size. */
-    if (fstat(gh->indexfd, &sb) < 0) {
+    if (fstat(fd, &sb) < 0) {
 	syslog(L_ERROR, "tradindexed: could not stat %s: %m", newidx);
 	close(fd);
 	OV3closegroup(gh, FALSE);
@@ -1019,7 +1014,7 @@ STATIC BOOL OV3packgroup(char *group, int delta) {
     }
 
     if (pwrite(fd, &gh->indexmem[ge->low - ge->base] , nbytes,
-	       sizeof(INDEXENTRY)*(ge->low - ge->base + delta - 1)) != nbytes) {
+	       sizeof(INDEXENTRY)*(ge->low - ge->base + delta)) != nbytes) {
 	syslog(L_ERROR, "tradindexed: packgroup cant write to %s: %m", newidx);
 	close(fd);
 	OV3closegroup(gh, FALSE);
@@ -1116,16 +1111,50 @@ BOOL tradindexed_expiregroup(char *group, int *lo) {
     newge.base = newge.low = newge.count = 0;
     while (ov3search(handle, &artnum, &data, &len, &token, &arrived, &expires)) {
 	ah = NULL;
-	if (!SMprobe(EXPENSIVESTAT, &token, NULL) || OVstatall) {
+	if (SMprobe(SELFEXPIRE, &token, NULL)) {
 	    if ((ah = SMretrieve(token, RETR_STAT)) == NULL)
 		continue;
-	    SMfreearticle(ah);
 	} else {
-	    if (!OVhisthasmsgid(data))
+	    if (!innconf->groupbaseexpiry && !OVhisthasmsgid(data))
 		continue; 
 	}
+	if (ah)
+	    SMfreearticle(ah);
 	if (innconf->groupbaseexpiry && OVgroupbasedexpire(token, group, data, len, arrived, expires))
 	    continue;
+#if 0
+	if (p = strchr(data, '\t')) {
+	    for (p--; (p >= data) && isdigit((int) *p); p--);
+	    if (p >= data) {
+		printf("bad article %s:%d\n", group, artnum);
+		sprintf(overdata, "%d\t", artnum);
+		i = strlen(overdata);
+		p = overdata + i;
+		memcpy(p, data, len - newlen - 2);
+		p = overdata + len - 2;
+		memcpy(p, "\r\n", 2);
+		OV3addrec(&newge, newgh, artnum, token, overdata, len, arrived, expires);
+		continue;
+	    }
+	}
+	if (atoi(data) != artnum) {
+	    printf("misnumbered article %s:%d\n", group, artnum);
+	    if ((p = strstr(data, "Xref: ")) == NULL) {
+		syslog(L_ERROR, "tradindexed: could not find Xref header in %s:%d", group, artnum);
+		continue;
+	    }
+	    if ((p = strchr(p, ' ')) == NULL) {
+		syslog(L_ERROR, "tradindexed: could not find space after Xref header in %s:%d", group, artnum);
+		continue;
+	    }
+	    if ((p = strstr(p, group)) == NULL) {
+		syslog(L_ERROR, "tradindexed: could not find group name in Xref header in %s:%d", group, artnum);
+		continue;
+	    }
+	    p += strlen(group) + 1;
+	    artnum = atoi(p);
+	}
+#endif
 	OV3addrec(&newge, newgh, artnum, token, data, len, arrived, expires);
     }
     do {
