@@ -1,28 +1,28 @@
-/*  $Id: timecaf.c 9870 2015-05-17 17:23:07Z iulius $
+/*  $Id: timecaf.c,v 1.10 1999/11/28 23:32:12 rra Exp $
 **
 **  Like the timehash storage method (and heavily inspired by it), but uses
 **  the CAF library to store multiple articles in a single file.
 */
-
 #include "config.h"
 #include "clibrary.h"
-#include "portable/mmap.h"
 #include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
-#include <fcntl.h>
+#include <syslog.h>
+#include <sys/mman.h>
 #include <sys/stat.h>
 #include <time.h>
 
+#ifdef HAVE_FCNTL_H
+# include <fcntl.h>
+#endif
+
 #include "caf.h"
-#include "inn/fdflag.h"
-#include "inn/innconf.h"
-#include "inn/messages.h"
-#include "inn/wire.h"
-#include "inn/libinn.h"
+#include "libinn.h"
+#include "macros.h"
 #include "methods.h"
 #include "timecaf.h"
-#include "inn/paths.h"
+#include "paths.h"
 
 /* Needed for htonl() and friends on AIX 4.1. */
 #include <netinet/in.h>
@@ -36,9 +36,9 @@ typedef struct {
     DIR	       		*sec; /* open handle on the 2nd level directory */
     DIR 		*ter; /* open handle on 3rd level dir. */
     struct dirent	*topde; /* last entry we got from top */
-    struct dirent	*secde; /* last entry we got from sec */
-    struct dirent	*terde; /* last entry we got from sec */
-    CAFTOCENT		*curtoc;
+    struct dirent	*secde; /* last entry we got from sec */ 
+    struct dirent	*terde; /* last entry we got from sec */ 
+    CAFTOCENT		*curtoc; 
     ARTNUM		curartnum;
     CAFHEADER		curheader;
 } PRIV_TIMECAF;
@@ -57,8 +57,8 @@ static unsigned int NumDeleteArtnums, MaxDeleteArtnums;
 typedef enum {FIND_DIR, FIND_CAF, FIND_TOPDIR} FINDTYPE;
 
 /*
-** Structures for the cache for stat information (to make expireover etc.)
-** faster.
+** Structures for the cache for stat information (to make expireover etc. 
+** faster. 
 **
 ** The first structure contains the TOC info for a single CAF file.  The 2nd
 ** one has pointers to the info for up to 256 CAF files, indexed
@@ -98,136 +98,89 @@ typedef struct caftocl3cache CAFTOCL3CACHE;
 static CAFTOCL3CACHE *TOCCache[256]; /* indexed by storage class! */
 static int TOCCacheHits, TOCCacheMisses;
 
-/*
-**  The token is @04nn00aabbccyyyyxxxx0000000000000000@
-**  where "04" is the timecaf method number,
-**  "nn" the hexadecimal value of the storage class,
-**  "aabbccdd" the arrival time in hexadecimal (dd is unused),
-**  "xxxxyyyy" the hexadecimal sequence number seqnum.
-**
-**  innconf->patharticles + '/timecaf-nn/bb/aacc.CF'
-**  where "nn" is the hexadecimal value of the storage class,
-**  "aabbccdd" the arrival time in hexadecimal (dd is unused).
-*/
-char *
-timecaf_explaintoken(const TOKEN token)
-{
-    char                *text;
-    uint32_t            arrival;
-    uint16_t            seqnum1;
-    uint16_t            seqnum2;
-
-    memcpy(&arrival, &token.token[0], sizeof(arrival));
-    memcpy(&seqnum1, &token.token[4], sizeof(seqnum1));
-    memcpy(&seqnum2, &token.token[6], sizeof(seqnum2));
- 
-    xasprintf(&text, "method=timecaf class=%u time=%lu seqnum=%lu file=%s/timecaf-%02x/%02x/%02x%02x.CF",
-              (unsigned int) token.class, ((unsigned long) (ntohl(arrival))) << 8,
-              ((unsigned long) ntohs(seqnum1)) + (((unsigned long) ntohs(seqnum2)) << 16),
-              innconf->patharticles, token.class,
-              (ntohl(arrival) >> 8) & 0xff,
-              (ntohl(arrival) >> 16) & 0xff,
-              ntohl(arrival) & 0xff);
-
-    return text;
-}
-
-static TOKEN MakeToken(time_t now, ARTNUM seqnum, STORAGECLASS class, TOKEN *oldtoken) {
+    
+static TOKEN MakeToken(time_t time, int seqnum, STORAGECLASS class, TOKEN *oldtoken) {
     TOKEN               token;
-    uint32_t            i;
-    uint16_t            s;
+    unsigned int        i;
+    unsigned short      s;
 
     if (oldtoken == (TOKEN *)NULL)
 	memset(&token, '\0', sizeof(token));
-    else
+    else 
 	memcpy(&token, oldtoken, sizeof(token));
     token.type = TOKEN_TIMECAF;
     token.class = class;
-    i = htonl(now);
+    i = htonl(time);
     memcpy(token.token, &i, sizeof(i));
-    s = htons(seqnum & 0xffff);
-    memcpy(&token.token[sizeof(i)], &s, sizeof(s));
-    s = htons((seqnum >> 16) & 0xffff);
-    memcpy(&token.token[sizeof(i)+sizeof(s)], &s, sizeof(s));
+    if (sizeof(i) > 4)
+	memmove(token.token, &token.token[sizeof(i) - 4], 4);
+    s = htons(seqnum);
+    memcpy(&token.token[4], &s + (sizeof(s) - 2), 2);
     return token;
 }
 
 
-static void BreakToken(TOKEN token, time_t *now, ARTNUM *seqnum) {
-    uint32_t            i;
-    uint16_t            s1 = 0;
-    uint16_t            s2 = 0;
+static void BreakToken(TOKEN token, int *time, int *seqnum) {
+    unsigned int        i;
+    unsigned short      s = 0;
 
     memcpy(&i, token.token, sizeof(i));
-    memcpy(&s1, &token.token[sizeof(i)], sizeof(s1));
-    memcpy(&s2, &token.token[sizeof(i)+sizeof(s1)], sizeof(s2));
-    *now = ntohl(i);
-    *seqnum = (ARTNUM)((ntohs(s2) << 16) + ntohs(s1));
+    memcpy(&s, &token.token[4], sizeof(s));
+    *time = ntohl(i);
+    *seqnum = (int)ntohs(s);
 }
 
-/*
+/* 
 ** Note: the time here is really "time>>8", i.e. a timestamp that's been
 ** shifted right by 8 bits.
 */
-static char *MakePath(time_t now, const STORAGECLASS class) {
+static char *MakePath(int time, const STORAGECLASS class) {
     char *path;
-    size_t length;
-
-    length = strlen(innconf->patharticles) + 32;
-    path = xmalloc(length);
-    snprintf(path, length, "%s/timecaf-%02x/%02x/%02x%02x.CF",
-             innconf->patharticles, class,
-             (unsigned int)((now >> 8) & 0xff),
-             (unsigned int)((now >> 16) & 0xff),
-             (unsigned int)(now & 0xff));
+    
+    /* innconf->patharticles + '/timecaf-zz/xx/xxxx.CF' */
+    path = NEW(char, strlen(innconf->patharticles) + 32);
+    sprintf(path, "%s/timecaf-%02x/%02x/%02x%02x.CF", innconf->patharticles,
+	    class, (time >> 8) & 0xff, (time >> 16) & 0xff, time & 0xff);
 
     return path;
 }
 
 static TOKEN *PathNumToToken(char *path, ARTNUM artnum) {
     int			n;
-    unsigned int        tclass, t1, t2;
-    STORAGECLASS        class;
-    time_t              timestamp;
+    unsigned int	t1, t2, class;
+    unsigned int	timestamp;
     static TOKEN	token;
 
-    n = sscanf(path, "timecaf-%02x/%02x/%04x.CF", &tclass, &t1, &t2);
+    n = sscanf(path, "timecaf-%02x/%02x/%04x.CF", &class, &t1, &t2);
     if (n != 3)
 	return (TOKEN *)NULL;
     timestamp = ((t1 << 8) & 0xff00) | ((t2 << 8) & 0xff0000) | ((t2 << 0) & 0xff);
-    class = tclass;
     token = MakeToken(timestamp, artnum, class, (TOKEN *)NULL);
     return &token;
 }
 
 
-bool timecaf_init(SMATTRIBUTE *attr) {
-    if (attr == NULL) {
-        warn("timecaf: attr is NULL");
-	SMseterror(SMERR_INTERNAL, "attr is NULL");
-	return false;
-    }
-    attr->selfexpire = false;
-    attr->expensivestat = false;
+BOOL timecaf_init(BOOL *selfexpire) {
+    *selfexpire = FALSE;
     if (STORAGE_TOKEN_LENGTH < 6) {
-        warn("timecaf: token length is less than six bytes");
+	syslog(L_FATAL, "timecaf: token length is less than 6 bytes");
 	SMseterror(SMERR_TOKENSHORT, NULL);
-	return false;
+	return FALSE;
     }
     ReadingFile.fd = WritingFile.fd = -1;
     ReadingFile.path = WritingFile.path = (char *)NULL;
-    return true;
+    return TRUE;
 }
 
 /*
 ** Routines for managing the 'TOC cache' (cache of TOCs of various CAF files)
 **
 ** Attempt to look up a given TOC entry in the cache.  Takes the timestamp
-** as arguments.
+** as arguments. 
 */
 
 static CAFTOCCACHEENT *
-CheckTOCCache(time_t timestamp, STORAGECLASS tokenclass)
+CheckTOCCache(int timestamp, int tokenclass)
 {
     CAFTOCL2CACHE *l2;
     CAFTOCL1CACHE *l1;
@@ -256,7 +209,7 @@ CheckTOCCache(time_t timestamp, STORAGECLASS tokenclass)
 ** cache.
 */
 static CAFTOCCACHEENT *
-AddTOCCache(time_t timestamp, CAFTOCENT *toc, CAFHEADER head, STORAGECLASS tokenclass)
+AddTOCCache(int timestamp, CAFTOCENT *toc, CAFHEADER head, int tokenclass)
 {
     CAFTOCL2CACHE *l2;
     CAFTOCL1CACHE *l1;
@@ -265,26 +218,26 @@ AddTOCCache(time_t timestamp, CAFTOCENT *toc, CAFHEADER head, STORAGECLASS token
     int i;
 
     if (TOCCache[tokenclass] == NULL) {
-	TOCCache[tokenclass] = xmalloc(sizeof(CAFTOCL3CACHE));
+	TOCCache[tokenclass] = NEW(CAFTOCL3CACHE, 1);
 	for (i = 0 ; i < 256 ; ++i) TOCCache[tokenclass]->l2ptr[i] = NULL;
     }
 
     tmp = (timestamp>>16) & 0xff;
     l2 = TOCCache[tokenclass]->l2ptr[tmp];
     if (l2 == NULL) {
-	TOCCache[tokenclass]->l2ptr[tmp] = l2 = xmalloc(sizeof(CAFTOCL2CACHE));
+	TOCCache[tokenclass]->l2ptr[tmp] = l2 = NEW(CAFTOCL2CACHE, 1);
 	for (i = 0 ; i < 256 ; ++i) l2->l1ptr[i] = NULL;
     }
 
     tmp = (timestamp>>8) & 0xff;
     l1 = l2->l1ptr[tmp];
     if (l1 == NULL) {
-	l2->l1ptr[tmp] = l1 = xmalloc(sizeof(CAFTOCL1CACHE));
+	l2->l1ptr[tmp] = l1 = NEW(CAFTOCL1CACHE, 1);
 	for (i = 0 ; i < 256 ; ++i) l1->entries[i] = NULL;
     }
 
     tmp = (timestamp) & 0xff;
-    cent = xmalloc(sizeof(CAFTOCCACHEENT));
+    cent = NEW(CAFTOCCACHEENT, 1);
     l1->entries[tmp] = cent;
 
     cent->header = head;
@@ -294,11 +247,11 @@ AddTOCCache(time_t timestamp, CAFTOCENT *toc, CAFHEADER head, STORAGECLASS token
 }
 
 /*
-** Do stating of an article, going thru the TOC cache if possible.
+** Do stating of an article, going thru the TOC cache if possible. 
 */
 
 static ARTHANDLE *
-StatArticle(time_t timestamp, ARTNUM artnum, STORAGECLASS tokenclass)
+StatArticle(int timestamp, ARTNUM artnum, int tokenclass)
 {
     CAFTOCCACHEENT *cent;
     CAFTOCENT *toc;
@@ -317,19 +270,19 @@ StatArticle(time_t timestamp, ARTNUM artnum, STORAGECLASS tokenclass)
 	    } else {
 		SMseterror(SMERR_UNDEFINED, NULL);
 	    }
-	    free(path);
+	    DISPOSE(path);
 	    return NULL;
 	}
 	cent = AddTOCCache(timestamp, toc, head, tokenclass);
-	free(path);
+	DISPOSE(path);
     }
-
+    
     /* check current TOC for the given artnum. */
     if (artnum < cent->header.Low || artnum > cent->header.High) {
 	SMseterror(SMERR_NOENT, NULL);
 	return NULL;
     }
-
+    
     tocentry = &(cent->toc[artnum - cent->header.Low]);
     if (tocentry->Size == 0) {
 	/* no article with that article number present */
@@ -338,7 +291,7 @@ StatArticle(time_t timestamp, ARTNUM artnum, STORAGECLASS tokenclass)
     }
 
     /* stat is a success, so build a null art struct to represent that. */
-    art = xmalloc(sizeof(ARTHANDLE));
+    art = NEW(ARTHANDLE, 1);
     art->type = TOKEN_TIMECAF;
     art->data = NULL;
     art->len = 0;
@@ -350,9 +303,9 @@ StatArticle(time_t timestamp, ARTNUM artnum, STORAGECLASS tokenclass)
 static void
 CloseOpenFile(CAFOPENFILE *foo) {
     if (foo->fd >= 0) {
-	close(foo->fd);
+	(void) close(foo->fd);
 	foo->fd = -1;
-	free(foo->path);
+	DISPOSE(foo->path);
 	foo->path = NULL;
     }
 }
@@ -361,10 +314,10 @@ TOKEN timecaf_store(const ARTHANDLE article, const STORAGECLASS class) {
     char                *path;
     char                *p;
     time_t              now;
-    time_t              timestamp;
+    int			timestamp;
     TOKEN               token;
     int                 fd;
-    ssize_t             result;
+    int                 result;
     ARTNUM		art;
 
     if (article.arrived == (time_t)0)
@@ -375,43 +328,38 @@ TOKEN timecaf_store(const ARTHANDLE article, const STORAGECLASS class) {
     timestamp = now>>8;
     art = 0;  /* magic: 0=="next available article number. */
 
-    memset(&token, 0, sizeof(token));
-
     path = MakePath(timestamp, class);
     /* check to see if we have this CAF file already open. */
     if (WritingFile.fd < 0 || strcmp(WritingFile.path, path) != 0) {
 	/* we're writing to a different file, close old one and start new one. */
 	CloseOpenFile(&WritingFile);
-	fd = CAFOpenArtWrite(path, &art, true, article.len);
+	fd = CAFOpenArtWrite(path, &art, TRUE, article.len);
 	if (fd < 0) {
 	    if (caf_error == CAF_ERR_IO && caf_errno == ENOENT) {
 		/* directories in the path don't exist, try creating them. */
 		p = strrchr(path, '/');
 		*p = '\0';
-		if (!MakeDirectory(path, true)) {
-                    syswarn("timecaf: could not create directory %s", path);
+		if (!MakeDirectory(path, TRUE)) {
+		    syslog(L_ERROR, "timecaf: could not make directory %s %m", path);
 		    token.type = TOKEN_EMPTY;
-		    free(path);
+		    DISPOSE(path);
 		    SMseterror(SMERR_UNDEFINED, NULL);
-		    token.type = TOKEN_EMPTY;
 		    return token;
 		} else {
 		    *p = '/';
-		    fd = CAFOpenArtWrite(path, &art, true, article.len);
+		    fd = CAFOpenArtWrite(path, &art, TRUE, article.len);
 		    if (fd < 0) {
-                        warn("timecaf: could not OpenArtWrite %s/%ld: %s",
-                             path, art, CAFErrorStr());
+			syslog(L_ERROR, "timecaf: could not OpenArtWrite %s/%d, %s", path, art, CAFErrorStr());
 			SMseterror(SMERR_UNDEFINED, NULL);
-			free(path);
+			DISPOSE(path);
 			token.type = TOKEN_EMPTY;
 			return token;
 		    }
-		}
+		} 
 	    } else {
-                warn("timecaf: could not OpenArtWrite %s/%ld: %s", path, art,
-                     CAFErrorStr());
+		syslog(L_ERROR, "timecaf: could not OpenArtWrite %s/%d, %s", path, art, CAFErrorStr());
 		SMseterror(SMERR_UNDEFINED, NULL);
-		free(path);
+		DISPOSE(path);
 		token.type = TOKEN_EMPTY;
 		return token;
 	    }
@@ -421,37 +369,35 @@ TOKEN timecaf_store(const ARTHANDLE article, const STORAGECLASS class) {
 	fd = WritingFile.fd;
 
 	/* nuke extraneous copy of path to avoid mem leaks. */
-	free(path);
+	DISPOSE(path);
 	path = WritingFile.path;
 
 	if (CAFStartWriteFd(fd, &art, article.len) < 0) {
-            warn("timecaf: could not OpenArtWriteFd %s/%ld: %s", path, art,
-                 CAFErrorStr());
+	    syslog(L_ERROR, "timecaf: could not OpenArtWrite %s/%d, %s", path, art, CAFErrorStr());
 	    SMseterror(SMERR_UNDEFINED, NULL);
-	    free(path);
+	    DISPOSE(path);
 	    token.type = TOKEN_EMPTY;
 	    return token;
 	}
     }
     WritingFile.fd = fd;
     WritingFile.path = path;
-    fdflag_close_exec(fd, true);
-    result = xwritev(fd, article.iov, article.iovcnt);
-    if (result != (ssize_t) article.len) {
+    CloseOnExec(fd, TRUE);
+    if ((result = write(fd, article.data, article.len)) != article.len) {
 	SMseterror(SMERR_UNDEFINED, NULL);
-        syswarn("timecaf: error writing %s", path);
+	syslog(L_ERROR, "timecaf error writing %s %m", path);
 	token.type = TOKEN_EMPTY;
 	CloseOpenFile(&WritingFile);
 	return token;
     }
-    if (CAFFinishArtWrite(fd) < 0) {
+    if (CAFFinishArtWrite(fd) < 0) { 
 	SMseterror(SMERR_UNDEFINED, NULL);
-        warn("timecaf: error writing %s: %s", path, CAFErrorStr());
+	syslog(L_ERROR, "timecaf error writing %s %s", path, CAFErrorStr());
 	token.type = TOKEN_EMPTY;
 	CloseOpenFile(&WritingFile);
 	return token;
     }
-
+    
     return MakeToken(timestamp, art, class, article.token);
 }
 
@@ -460,21 +406,10 @@ static ARTHANDLE *OpenArticle(const char *path, ARTNUM artnum, const RETRTYPE am
     int                 fd;
     PRIV_TIMECAF        *private;
     char                *p;
-    size_t		len;
+    SIZE_T		len;
     ARTHANDLE           *art;
-    static long		pagesize = 0;
-
-    if (pagesize == 0) {
-        pagesize = getpagesize();
-        if (pagesize < 0) {
-            syswarn("timecaf: getpagesize failed");
-            pagesize = 0;
-	    return NULL;
-        }
-    }
-
 /* XXX need to figure some way to cache open fds or something? */
-    if ((fd = CAFOpenArtRead(path, artnum, &len)) < 0) {
+    if ((fd = CAFOpenArtRead((char *)path, artnum, &len)) < 0) {
         if (caf_error == CAF_ERR_ARTNOTHERE) {
 	    SMseterror(SMERR_NOENT, NULL);
 	} else {
@@ -483,7 +418,7 @@ static ARTHANDLE *OpenArticle(const char *path, ARTNUM artnum, const RETRTYPE am
 	return NULL;
     }
 
-    art = xmalloc(sizeof(ARTHANDLE));
+    art = NEW(ARTHANDLE, 1);
     art->type = TOKEN_TIMECAF;
 
     if (amount == RETR_STAT) {
@@ -494,7 +429,7 @@ static ARTHANDLE *OpenArticle(const char *path, ARTNUM artnum, const RETRTYPE am
 	return art;
     }
 
-    private = xmalloc(sizeof(PRIV_TIMECAF));
+    private = NEW(PRIV_TIMECAF, 1);
     art->private = (void *)private;
     private->artlen = len;
     if (innconf->articlemmap) {
@@ -502,30 +437,32 @@ static ARTHANDLE *OpenArticle(const char *path, ARTNUM artnum, const RETRTYPE am
 	size_t delta;
 
 	curoff = lseek(fd, (off_t) 0, SEEK_CUR);
-	delta = curoff % pagesize;
+	delta = curoff % getpagesize();
 	tmpoff = curoff - delta;
 	private->mmaplen = len + delta;
-	if ((private->mmapbase = mmap(NULL, private->mmaplen, PROT_READ, MAP_SHARED, fd, tmpoff)) == MAP_FAILED) {
+	if ((private->mmapbase = mmap((MMAP_PTR)0, private->mmaplen, PROT_READ, MAP__ARG, fd, tmpoff)) == (MMAP_PTR)-1) {
 	    SMseterror(SMERR_UNDEFINED, NULL);
-            syswarn("timecaf: could not mmap article");
-	    free(art->private);
-	    free(art);
+	    syslog(L_ERROR, "timecaf: could not mmap article: %m");
+	    DISPOSE(art->private);
+	    DISPOSE(art);
 	    return NULL;
 	}
-	mmap_invalidate(private->mmapbase, private->mmaplen);
-        if (amount == RETR_ALL)
-            madvise(private->mmapbase, private->mmaplen, MADV_WILLNEED);
-        else
-            madvise(private->mmapbase, private->mmaplen, MADV_SEQUENTIAL);
+#ifdef MMAP_MISSES_WRITES
+# ifdef HAVE_MSYNC_3_ARG
+	msync(private->mmapbase, private->mmaplen, MS_INVALIDATE);
+# else
+        msync(private->mmapbase, private->mmaplen);
+# endif
+#endif
 	private->artdata = private->mmapbase + delta;
     } else {
-        private->artdata = xmalloc(private->artlen);
+        private->artdata = NEW(char, private->artlen);
 	if (read(fd, private->artdata, private->artlen) < 0) {
 	    SMseterror(SMERR_UNDEFINED, NULL);
-            syswarn("timecaf: could not read article");
-	    free(private->artdata);
-	    free(art->private);
-	    free(art);
+	    syslog(L_ERROR, "timecaf: could not read article: %m");
+	    DISPOSE(private->artdata);
+	    DISPOSE(art->private);
+	    DISPOSE(art);
 	    return NULL;
 	}
     }
@@ -539,29 +476,31 @@ static ARTHANDLE *OpenArticle(const char *path, ARTNUM artnum, const RETRTYPE am
     private->topde = NULL;
     private->secde = NULL;
     private->terde = NULL;
-
+    
     if (amount == RETR_ALL) {
 	art->data = private->artdata;
 	art->len = private->artlen;
 	return art;
     }
-
-    if ((p = wire_findbody(private->artdata, private->artlen)) == NULL) {
+    
+    if ((p = SMFindBody(private->artdata, private->artlen)) == NULL) {
 	SMseterror(SMERR_NOBODY, NULL);
-	if (innconf->articlemmap)
+	if (innconf->articlemmap) {
+#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
+	    madvise(private->mmapbase, private->mmaplen, MADV_DONTNEED);
+#endif
 	    munmap(private->mmapbase, private->mmaplen);
-	else
-	    free(private->artdata);
-	free(art->private);
-	free(art);
+	} else {
+	    DISPOSE(private->artdata);
+	}
+	DISPOSE(art->private);
+	DISPOSE(art);
 	return NULL;
     }
 
     if (amount == RETR_HEAD) {
 	art->data = private->artdata;
 	art->len = p - private->artdata;
-        /* Headers end just before the first empty line (\r\n). */
-        art->len = art->len - 2;
 	return art;
     }
 
@@ -571,23 +510,27 @@ static ARTHANDLE *OpenArticle(const char *path, ARTNUM artnum, const RETRTYPE am
 	return art;
     }
     SMseterror(SMERR_UNDEFINED, "Invalid retrieve request");
-    if (innconf->articlemmap)
+    if (innconf->articlemmap) {
+#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
+	madvise(private->mmapbase, private->mmaplen, MADV_DONTNEED);
+#endif
 	munmap(private->mmapbase, private->mmaplen);
-    else
-	free(private->artdata);
-    free(art->private);
-    free(art);
+    } else {
+	DISPOSE(private->artdata);
+    }
+    DISPOSE(art->private);
+    DISPOSE(art);
     return NULL;
 }
 
 ARTHANDLE *timecaf_retrieve(const TOKEN token, const RETRTYPE amount) {
-    time_t              timestamp;
-    ARTNUM              artnum;
+    int                 timestamp;
+    int			artnum;
     char                *path;
     ARTHANDLE           *art;
     static TOKEN	ret_token;
     time_t		now;
-
+    
     if (token.type != TOKEN_TIMECAF) {
 	SMseterror(SMERR_INTERNAL, NULL);
 	return NULL;
@@ -596,15 +539,15 @@ ARTHANDLE *timecaf_retrieve(const TOKEN token, const RETRTYPE amount) {
     BreakToken(token, &timestamp, &artnum);
 
     /*
-    ** Do a possible shortcut on RETR_STAT requests, going through the "TOC cache"
-    ** we mentioned above.  We only try to go through the TOC Cache under these
+    ** Do a possible shortcut on RETR_STAT requests, going thru the "TOC cache"
+    ** we mentioned above.  We only try to go thru the TOC Cache under these
     ** conditions:
-    **   1) SMpreopen is true (so we're "preopening" the TOCs).
+    **   1) SMpreopen is TRUE (so we're "preopening" the TOCs.)
     **   2) the timestamp is older than the timestamp corresponding to current
-    ** time.  Any timestamp that matches current time (to within 256 seconds)
-    ** would be in a CAF file that innd is actively
+    ** time. Any timestamp that matches current time (to within 256 secondsf
+    ** would be in a CAF file that innd is actively 
     ** writing, in which case we would not want to cache the TOC for that
-    ** CAF file.
+    ** CAF file. 
     */
 
     if (SMpreopen && amount == RETR_STAT) {
@@ -617,13 +560,13 @@ ARTHANDLE *timecaf_retrieve(const TOKEN token, const RETRTYPE amount) {
     path = MakePath(timestamp, token.class);
     if ((art = OpenArticle(path, artnum, amount)) != (ARTHANDLE *)NULL) {
 	art->arrived = timestamp<<8; /* XXX not quite accurate arrival time,
-				     ** but getting a more accurate one would
+				     ** but getting a more accurate one would 
 				     ** require more fiddling with CAF innards.
 				     */
 	ret_token = token;
 	art->token = &ret_token;
     }
-    free(path);
+    DISPOSE(path);
     return art;
 }
 
@@ -632,24 +575,28 @@ void timecaf_freearticle(ARTHANDLE *article) {
 
     if (!article)
 	return;
-
+    
     if (article->private) {
 	private = (PRIV_TIMECAF *)article->private;
-	if (innconf->articlemmap)
+	if (innconf->articlemmap) {
+#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
+	    madvise(private->mmapbase, private->mmaplen, MADV_DONTNEED);
+#endif
 	    munmap(private->mmapbase, private->mmaplen);
-	else
-	    free(private->artdata);
+	} else {
+	    DISPOSE(private->artdata);
+	}
 	if (private->top)
 	    closedir(private->top);
 	if (private->sec)
 	    closedir(private->sec);
 	if (private->ter)
 	    closedir(private->ter);
-	if (private->curtoc)
-	    free(private->curtoc);
-	free(private);
+	if (private->curtoc) 
+	    DISPOSE(private->curtoc);
+	DISPOSE(private);
     }
-    free(article);
+    DISPOSE(article);
 }
 
 /* Do cancels of all the article ids collected for a given pathname. */
@@ -658,7 +605,7 @@ static void
 DoCancels(void) {
     if (DeletePath != NULL) {
 	if (NumDeleteArtnums != 0) {
-	    /*
+	    /* 
 	    ** Murgle. If we are trying to cancel something out of the
 	    ** currently open-for-writing file, we need to close it before
 	    ** doing CAFRemove...
@@ -667,23 +614,23 @@ DoCancels(void) {
 	        CloseOpenFile(&WritingFile);
 	    }
 	    /* XXX should really check err. code here, but not much we can really do. */
-	    CAFRemoveMultArts(DeletePath, NumDeleteArtnums, DeleteArtnums);
-	    free(DeleteArtnums);
+	    (void)CAFRemoveMultArts(DeletePath, NumDeleteArtnums, DeleteArtnums);
+	    DISPOSE(DeleteArtnums);
 	    DeleteArtnums = NULL;
 	    NumDeleteArtnums = MaxDeleteArtnums = 0;
 	}
-	free(DeletePath);
+	DISPOSE(DeletePath);
 	DeletePath = NULL;
     }
 }
-
-bool timecaf_cancel(TOKEN token) {
-    time_t              now;
-    ARTNUM              seqnum;
+	    
+BOOL timecaf_cancel(TOKEN token) {
+    int                 time;
+    int                 seqnum;
     char                *path;
 
-    BreakToken(token, &now, &seqnum);
-    path = MakePath(now, token.class);
+    BreakToken(token, &time, &seqnum);
+    path = MakePath(time, token.class);
     if (DeletePath == NULL) {
 	DeletePath = path;
     } else if (strcmp(DeletePath, path) != 0) {
@@ -691,7 +638,7 @@ bool timecaf_cancel(TOKEN token) {
 	DoCancels();
 	DeletePath = path;
     } else {
-	free(path); /* free redundant copy of path */
+	DISPOSE(path); /* free redundant copy of path */
     }
     if (NumDeleteArtnums >= MaxDeleteArtnums) {
 	/* allocate/expand storage for artnums. */
@@ -700,36 +647,34 @@ bool timecaf_cancel(TOKEN token) {
 	} else {
 	    MaxDeleteArtnums *= 2;
 	}
-        DeleteArtnums = xrealloc(DeleteArtnums, MaxDeleteArtnums * sizeof(ARTNUM));
+	RENEW(DeleteArtnums, ARTNUM, MaxDeleteArtnums);
     }
     DeleteArtnums[NumDeleteArtnums++] = seqnum;	
 
-    return true;
+    return TRUE;
 }
 
 static struct dirent *FindDir(DIR *dir, FINDTYPE type) {
     struct dirent       *de;
-
+    
     while ((de = readdir(dir)) != NULL) {
         if (type == FIND_TOPDIR)
 	    if ((strlen(de->d_name) == 10) &&
 		(strncmp(de->d_name, "timecaf-", 8) == 0) &&
-		isxdigit((unsigned char) de->d_name[8]) &&
-		isxdigit((unsigned char) de->d_name[9]))
+		isxdigit(de->d_name[8]) &&
+		isxdigit(de->d_name[9]))
 	        return de;
 
 	if (type == FIND_DIR)
-	    if ((strlen(de->d_name) == 2)
-                && isxdigit((unsigned char) de->d_name[0])
-                && isxdigit((unsigned char) de->d_name[1]))
+	    if ((strlen(de->d_name) == 2) && isxdigit(de->d_name[0]) && isxdigit(de->d_name[1]))
 		return de;
 
 	if (type == FIND_CAF)
 	    if ((strlen(de->d_name) == 7) &&
-		isxdigit((unsigned char) de->d_name[0]) &&
-		isxdigit((unsigned char) de->d_name[1]) &&
-		isxdigit((unsigned char) de->d_name[2]) &&
-		isxdigit((unsigned char) de->d_name[3]) &&
+		isxdigit(de->d_name[0]) &&
+		isxdigit(de->d_name[1]) &&
+		isxdigit(de->d_name[2]) &&
+		isxdigit(de->d_name[3]) &&
 		(de->d_name[4] == '.') &&
 		(de->d_name[5] == 'C') &&
 		(de->d_name[6] == 'F'))
@@ -740,7 +685,7 @@ static struct dirent *FindDir(DIR *dir, FINDTYPE type) {
 }
 
 /* Grovel thru a CAF table-of-contents finding the next still-existing article */
-static int
+int
 FindNextArt(const CAFHEADER *head, CAFTOCENT *toc, ARTNUM *artp)
 {
     ARTNUM art;
@@ -750,29 +695,26 @@ FindNextArt(const CAFHEADER *head, CAFTOCENT *toc, ARTNUM *artp)
 	art = head->Low - 1; /* we never use art # 0, so 0 is a flag to start
 			       searching at the beginning */
     }
-    while (true) {
+    while (TRUE) {
 	art++;
-	if (art > head->High) return false; /* ran off the end of the TOC */
+	if (art > head->High) return FALSE; /* ran off the end of the TOC */
 	tocp = &toc[art - head->Low];
 	if (tocp->Size != 0) {
 	    /* got a valid article */
 	    *artp = art;
-	    return true;
+	    return TRUE;
 	}
     }
 }
 
 
-ARTHANDLE *
-timecaf_next(ARTHANDLE *article, const RETRTYPE amount)
-{
+
+ARTHANDLE *timecaf_next(const ARTHANDLE *article, const RETRTYPE amount) {
     PRIV_TIMECAF	priv, *newpriv;
     char                *path;
     ARTHANDLE           *art;
-    size_t              length;
 
-    length = strlen(innconf->patharticles) + 32;
-    path = xmalloc(length);
+    path = NEW(char, strlen(innconf->patharticles) + 32);
     if (article == NULL) {
 	priv.top = NULL;
 	priv.sec = NULL;
@@ -783,17 +725,21 @@ timecaf_next(ARTHANDLE *article, const RETRTYPE amount)
 	priv.terde = NULL;
     } else {
 	priv = *(PRIV_TIMECAF *)article->private;
-	free(article->private);
-	free(article);
-	if (innconf->articlemmap)
+	DISPOSE(article->private);
+	DISPOSE((void *)article);
+	if (innconf->articlemmap) {
+#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
+	    madvise(priv.mmapbase, priv.mmaplen, MADV_DONTNEED);
+#endif
 	    munmap(priv.mmapbase, priv.mmaplen);
-	else
-	    free(priv.artdata);
+	} else {
+	    DISPOSE(priv.artdata);
+	}
     }
 
     while (priv.curtoc == NULL || !FindNextArt(&priv.curheader, priv.curtoc, &priv.curartnum)) {
 	if (priv.curtoc) {
-	    free(priv.curtoc);
+	    DISPOSE(priv.curtoc);
 	    priv.curtoc = NULL;
 	}
 	while (!priv.ter || ((priv.terde = FindDir(priv.ter, FIND_CAF)) == NULL)) {
@@ -811,43 +757,43 @@ timecaf_next(ARTHANDLE *article, const RETRTYPE amount)
 			/* end of search */
 			closedir(priv.top);
 			priv.top = NULL;
-			free(path);
+			DISPOSE(path);
 			return NULL;
 		    }
-		    snprintf(path, length, "%s", innconf->patharticles);
+		    sprintf(path, "%s", innconf->patharticles);
 		    if ((priv.top = opendir(path)) == NULL) {
 			SMseterror(SMERR_UNDEFINED, NULL);
-			free(path);
+			DISPOSE(path);
 			return NULL;
 		    }
 		    if ((priv.topde = FindDir(priv.top, FIND_TOPDIR)) == NULL) {
 			SMseterror(SMERR_UNDEFINED, NULL);
 			closedir(priv.top);
-			free(path);
+			DISPOSE(path);
 			return NULL;
 		    }
 		}
-		snprintf(path, length, "%s/%s", innconf->patharticles, priv.topde->d_name);
+		sprintf(path, "%s/%s", innconf->patharticles, priv.topde->d_name);
 		if ((priv.sec = opendir(path)) == NULL)
 		    continue;
 	    }
-	    snprintf(path, length, "%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name);
+	    sprintf(path, "%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name);
 	    if ((priv.ter = opendir(path)) == NULL)
 		continue;
 	}
-	snprintf(path, length, "%s/%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name, priv.terde->d_name);
+	sprintf(path, "%s/%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name, priv.terde->d_name);
 	if ((priv.curtoc = CAFReadTOC(path, &priv.curheader)) == NULL)
 	    continue;
 	priv.curartnum = 0;
     }
-    snprintf(path, length, "%s/%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name, priv.terde->d_name);
+    sprintf(path, "%s/%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name, priv.terde->d_name);
     art = OpenArticle(path, priv.curartnum, amount);
     if (art == (ARTHANDLE *)NULL) {
-	art = xmalloc(sizeof(ARTHANDLE));
+	art = NEW(ARTHANDLE, 1);
 	art->type = TOKEN_TIMECAF;
 	art->data = NULL;
 	art->len = 0;
-	art->private = xmalloc(sizeof(PRIV_TIMECAF));
+	art->private = (void *)NEW(PRIV_TIMECAF, 1);
     }
     newpriv = (PRIV_TIMECAF *)art->private;
     newpriv->top = priv.top;
@@ -859,40 +805,33 @@ timecaf_next(ARTHANDLE *article, const RETRTYPE amount)
     newpriv->curheader = priv.curheader;
     newpriv->curtoc = priv.curtoc;
     newpriv->curartnum = priv.curartnum;
-
-    snprintf(path, length, "%s/%s/%s", priv.topde->d_name, priv.secde->d_name, priv.terde->d_name);
+    
+    sprintf(path, "%s/%s/%s", priv.topde->d_name, priv.secde->d_name, priv.terde->d_name);
     art->token = PathNumToToken(path, priv.curartnum);
     art->arrived = priv.curtoc[priv.curartnum - priv.curheader.Low].ModTime;
-    free(path);
+    DISPOSE(path);
     return art;
 }
 
-bool timecaf_ctl(PROBETYPE type, TOKEN *token UNUSED, void *value) {
+BOOL timecaf_ctl(PROBETYPE type, TOKEN *token, void *value) {
     struct artngnum *ann;
 
     switch (type) {
     case SMARTNGNUM:
 	if ((ann = (struct artngnum *)value) == NULL)
-	    return false;
+	    return FALSE;
 	/* make SMprobe() call timecaf_retrieve() */
 	ann->artnum = 0;
-	return true;
+	return TRUE;
     default:
-	return false;
+	return FALSE;
     }
 }
 
-bool timecaf_flushcacheddata(FLUSHTYPE type) {
-    if (type == SM_ALL || type == SM_CANCELLEDART)
+BOOL timecaf_flushcacheddata(FLUSHTYPE type) {
+    if (type == SM_ALL || type == SM_CANCELEDART)
 	DoCancels();
-    return true;
-}
-
-void
-timecaf_printfiles(FILE *file, TOKEN token, char **xref UNUSED,
-                   int ngroups UNUSED)
-{
-    fprintf(file, "%s\n", TokenToText(token));
+    return TRUE;
 }
 
 void timecaf_shutdown(void) {
