@@ -1,25 +1,26 @@
-/*  $Id: timehash.c 9870 2015-05-17 17:23:07Z iulius $
+/*  $Id: timehash.c,v 1.34 1999/11/19 09:29:33 rra Exp $
 **
-**  Storage manager module for timehash method.
+**  Timehash based storage method.
 */
-
 #include "config.h"
 #include "clibrary.h"
-#include "portable/mmap.h"
 #include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
-#include <fcntl.h>
 #include <netinet/in.h>
+#include <syslog.h>
+#include <sys/mman.h>
 #include <sys/stat.h>
 #include <time.h>
 
-#include "inn/innconf.h"
-#include "inn/messages.h"
-#include "inn/wire.h"
-#include "inn/libinn.h"
+#ifdef HAVE_FCNTL_H
+# include <fcntl.h>
+#endif
+
+#include "libinn.h"
+#include "macros.h"
 #include "methods.h"
-#include "inn/paths.h"
+#include "paths.h"
 #include "timehash.h"
 
 typedef struct {
@@ -27,7 +28,7 @@ typedef struct {
     int                 len;      /* Length of the file */
     DIR                 *top;     /* Open handle on the top level directory */
     DIR                 *sec;     /* Open handle on the 2nd level directory */
-    DIR                 *ter;     /* Open handle on the third level directory */
+    DIR			*ter;     /* Open handle on the third level directory */
     DIR                 *artdir;  /* Open handle on the article directory */
     struct dirent       *topde;   /* dirent entry for the last entry retrieved in top */
     struct dirent       *secde;   /* dirent entry for the last entry retrieved in sec */
@@ -38,111 +39,69 @@ typedef enum {FIND_DIR, FIND_ART, FIND_TOPDIR} FINDTYPE;
 
 static int SeqNum = 0;
 
-/*
-**  The token is @02nnaabbccddyyyy00000000000000000000@
-**  where "02" is the timehash method number,
-**  "nn" the hexadecimal value of the storage class,
-**  "aabbccdd" the arrival time in hexadecimal,
-**  "yyyy" the hexadecimal sequence number seqnum.
-**
-**  innconf->patharticles + '/time-nn/bb/cc/yyyy-aadd'
-**  where "nn" is the hexadecimal value of the storage class,
-**  "aabbccdd" the arrival time in hexadecimal,
-**  "yyyy" the hexadecimal sequence number seqnum.
-*/
-char *
-timehash_explaintoken(const TOKEN token)
-{
-    char                *text;
-    uint32_t            arrival;
-    uint16_t            seqnum;
-
-    memcpy(&arrival, &token.token[0], sizeof(arrival));
-    memcpy(&seqnum, &token.token[4], sizeof(seqnum));
-
-    xasprintf(&text, "method=timehash class=%u time=%lu seqnum=%lu file=%s/time-%02x/%02x/%02x/%04x-%02x%02x",
-              (unsigned int) token.class, (unsigned long) ntohl(arrival), (unsigned long) ntohs(seqnum),
-              innconf->patharticles, token.class, (ntohl(arrival) >> 16) & 0xff,
-              (ntohl(arrival) >> 8) & 0xff, ntohs(seqnum),
-              (ntohl(arrival) >> 24) & 0xff, ntohl(arrival) & 0xff);
-
-    return text;
-}
-
-static TOKEN MakeToken(time_t now, int seqnum, STORAGECLASS class, TOKEN *oldtoken) {
+static TOKEN MakeToken(time_t time, int seqnum, STORAGECLASS class, TOKEN *oldtoken) {
     TOKEN               token;
-    uint32_t            i;
-    uint16_t            s;
+    unsigned int        i;
+    unsigned short      s;
 
     if (oldtoken == (TOKEN *)NULL)
 	memset(&token, '\0', sizeof(token));
-    else
+    else 
 	memcpy(&token, oldtoken, sizeof(token));
     token.type = TOKEN_TIMEHASH;
     token.class = class;
-    i = htonl(now);
+    i = htonl(time);
     memcpy(token.token, &i, sizeof(i));
-    s = htons(seqnum & 0xffff);
-    memcpy(&token.token[sizeof(i)], &s, sizeof(s));
+    if (sizeof(i) > 4)
+	memmove(token.token, &token.token[sizeof(i) - 4], 4);
+    s = htons(seqnum);
+    memcpy(&token.token[4], &s + (sizeof(s) - 2), 2);
     return token;
 }
 
-static void BreakToken(TOKEN token, time_t *now, int *seqnum) {
-    uint32_t            i;
-    uint16_t            s = 0;
+static void BreakToken(TOKEN token, int *time, int *seqnum) {
+    unsigned int        i;
+    unsigned short      s = 0;
 
     memcpy(&i, token.token, sizeof(i));
-    memcpy(&s, &token.token[sizeof(i)], sizeof(s));
-    *now = ntohl(i);
+    memcpy(&s, &token.token[4], sizeof(s));
+    *time = ntohl(i);
     *seqnum = (int)ntohs(s);
 }
 
-static char *MakePath(time_t now, int seqnum, const STORAGECLASS class) {
+static char *MakePath(int time, int seqnum, const STORAGECLASS class) {
     char *path;
-    size_t length;
-
-    length = strlen(innconf->patharticles) + 32;
-    path = xmalloc(length);
-    snprintf(path, length, "%s/time-%02x/%02x/%02x/%04x-%04x",
-             innconf->patharticles, class,
-             (unsigned int)((now >> 16) & 0xff),
-             (unsigned int)((now >> 8) & 0xff),
-             seqnum,
-             (unsigned int)((now & 0xff) | ((now >> 16 & 0xff00))));
+    
+    /* innconf->patharticles + '/time-zz/xx/xx/yyyy-xxxx' */
+    path = NEW(char, strlen(innconf->patharticles) + 32);
+    sprintf(path, "%s/time-%02x/%02x/%02x/%04x-%04x", innconf->patharticles,
+	    class, (time >> 16) & 0xff, (time >> 8) & 0xff, seqnum,
+	    (time & 0xff) | ((time >> 16 & 0xff00)));
     return path;
 }
 
 static TOKEN *PathToToken(char *path) {
     int			n;
-    unsigned int        tclass, t1, t2, t3, seqnum;
-    STORAGECLASS        class;
-    time_t		now;
+    unsigned int	t1, t2, t3, seqnum, class;
+    time_t		time;
     static TOKEN	token;
 
-    n = sscanf(path, "time-%02x/%02x/%02x/%04x-%04x",
-               &tclass, &t1, &t2, &seqnum, &t3);
+    n = sscanf(path, "time-%02x/%02x/%02x/%04x-%04x", &class, &t1, &t2, &seqnum, &t3);
     if (n != 5)
 	return (TOKEN *)NULL;
-    now = ((t1 << 16) & 0xff0000) | ((t2 << 8) & 0xff00) | ((t3 << 16) & 0xff000000) | (t3 & 0xff);
-    class = tclass;
-    token = MakeToken(now, seqnum, class, (TOKEN *)NULL);
+    time = ((t1 << 16) & 0xff0000) | ((t2 << 8) & 0xff00) | ((t3 << 16) & 0xff000000) | (t3 & 0xff);
+    token = MakeToken(time, seqnum, class, (TOKEN *)NULL);
     return &token;
 }
 
-bool timehash_init(SMATTRIBUTE *attr) {
-    if (attr == NULL) {
-        warn("timehash: attr is NULL");
-	SMseterror(SMERR_INTERNAL, "attr is NULL");
-	return false;
-    }
-    attr->selfexpire = false;
-    attr->expensivestat = true;
+BOOL timehash_init(BOOL *selfexpire) {
+    *selfexpire = FALSE;
     if (STORAGE_TOKEN_LENGTH < 6) {
-        warn("timehash: token length is less than six bytes");
+	syslog(L_FATAL, "timehash: token length is less than 6 bytes");
 	SMseterror(SMERR_TOKENSHORT, NULL);
-	return false;
+	return FALSE;
     }
-    return true;
+    return TRUE;
 }
 
 TOKEN timehash_store(const ARTHANDLE article, const STORAGECLASS class) {
@@ -151,7 +110,7 @@ TOKEN timehash_store(const ARTHANDLE article, const STORAGECLASS class) {
     time_t              now;
     TOKEN               token;
     int                 fd;
-    ssize_t             result;
+    int                 result;
     int                 seq;
     int                 i;
 
@@ -160,8 +119,6 @@ TOKEN timehash_store(const ARTHANDLE article, const STORAGECLASS class) {
     else
 	now = article.arrived;
 
-    memset(&token, 0, sizeof(token));
-
     for (i = 0; i < 0x10000; i++) {
 	seq = SeqNum;
 	SeqNum = (SeqNum + 1) & 0xffff;
@@ -172,19 +129,19 @@ TOKEN timehash_store(const ARTHANDLE article, const STORAGECLASS class) {
 		continue;
 	    p = strrchr(path, '/');
 	    *p = '\0';
-	    if (!MakeDirectory(path, true)) {
-                syswarn("timehash: could not make directory %s", path);
+	    if (!MakeDirectory(path, TRUE)) {
+	        syslog(L_ERROR, "timehash: could not make directory %s %m", path);
 	        token.type = TOKEN_EMPTY;
-	        free(path);
+	        DISPOSE(path);
 	        SMseterror(SMERR_UNDEFINED, NULL);
 	        return token;
 	    } else {
 	        *p = '/';
 	        if ((fd = open(path, O_CREAT|O_EXCL|O_WRONLY, ARTFILE_MODE)) < 0) {
 		    SMseterror(SMERR_UNDEFINED, NULL);
-                    syswarn("timehash: could not create %s", path);
+		    syslog(L_ERROR, "timehash: could not open %s %m", path);
 		    token.type = TOKEN_EMPTY;
-		    free(path);
+		    DISPOSE(path);
 		    return token;
 	        }
 	    }
@@ -193,25 +150,23 @@ TOKEN timehash_store(const ARTHANDLE article, const STORAGECLASS class) {
     }
     if (i == 0x10000) {
 	SMseterror(SMERR_UNDEFINED, NULL);
-        warn("timehash: all sequence numbers for time %lu and class %d are"
-             " reserved", (unsigned long) now, class);
+	syslog(L_ERROR, "timehash: all sequence numbers for the time and class are reserved %d %d", now, class);
 	token.type = TOKEN_EMPTY;
-	free(path);
+	DISPOSE(path);
 	return token;
     }
 
-    result = xwritev(fd, article.iov, article.iovcnt);
-    if (result != (ssize_t) article.len) {
+    if ((result = write(fd, article.data, article.len)) != article.len) {
 	SMseterror(SMERR_UNDEFINED, NULL);
-        syswarn("timehash: error writing %s", path);
+	syslog(L_ERROR, "timehash error writing %s %m", path);
 	close(fd);
 	token.type = TOKEN_EMPTY;
 	unlink(path);
-	free(path);
+	DISPOSE(path);
 	return token;
     }
     close(fd);
-    free(path);
+    DISPOSE(path);
     return MakeToken(now, seq, class, article.token);
 }
 
@@ -227,7 +182,7 @@ static ARTHANDLE *OpenArticle(const char *path, RETRTYPE amount) {
             SMseterror(SMERR_UNDEFINED, NULL);
             return NULL;
         }
-        art = xmalloc(sizeof(ARTHANDLE));
+        art = NEW(ARTHANDLE, 1);
         art->type = TOKEN_TIMEHASH;
 	art->data = NULL;
 	art->len = 0;
@@ -240,39 +195,35 @@ static ARTHANDLE *OpenArticle(const char *path, RETRTYPE amount) {
 	return NULL;
     }
 
-    art = xmalloc(sizeof(ARTHANDLE));
+    art = NEW(ARTHANDLE, 1);
     art->type = TOKEN_TIMEHASH;
 
     if (fstat(fd, &sb) < 0) {
 	SMseterror(SMERR_UNDEFINED, NULL);
-        syswarn("timehash: could not fstat article");
-	free(art);
+	syslog(L_ERROR, "timehash: could not fstat article: %m");
+	DISPOSE(art);
 	return NULL;
     }
-
-    private = xmalloc(sizeof(PRIV_TIMEHASH));
+    
+    private = NEW(PRIV_TIMEHASH, 1);
     art->private = (void *)private;
     private->len = sb.st_size;
     if (innconf->articlemmap) {
-	if ((private->base = mmap(NULL, sb.st_size, PROT_READ, MAP_SHARED, fd, 0)) == MAP_FAILED) {
+	if ((private->base = mmap((MMAP_PTR)0, sb.st_size, PROT_READ, MAP__ARG, fd, 0)) == (MMAP_PTR)-1) {
 	    SMseterror(SMERR_UNDEFINED, NULL);
-            syswarn("timehash: could not mmap article");
-	    free(art->private);
-	    free(art);
+	    syslog(L_ERROR, "timehash: could not mmap article: %m");
+	    DISPOSE(art->private);
+	    DISPOSE(art);
 	    return NULL;
 	}
-        if (amount == RETR_ALL)
-            madvise(private->base, sb.st_size, MADV_WILLNEED);
-        else
-            madvise(private->base, sb.st_size, MADV_SEQUENTIAL);
     } else {
-	private->base = xmalloc(private->len);
+	private->base = NEW(char, private->len);
 	if (read(fd, private->base, private->len) < 0) {
 	    SMseterror(SMERR_UNDEFINED, NULL);
-            syswarn("timehash: could not read article");
-	    free(private->base);
-	    free(art->private);
-	    free(art);
+	    syslog(L_ERROR, "timehash: could not read article: %m");
+	    DISPOSE(private->base);
+	    DISPOSE(art->private);
+	    DISPOSE(art);
 	    return NULL;
 	}
     }
@@ -285,29 +236,31 @@ static ARTHANDLE *OpenArticle(const char *path, RETRTYPE amount) {
     private->topde = NULL;
     private->secde = NULL;
     private->terde = NULL;
-
+    
     if (amount == RETR_ALL) {
 	art->data = private->base;
 	art->len = private->len;
 	return art;
     }
-
-    if ((p = wire_findbody(private->base, private->len)) == NULL) {
+    
+    if ((p = SMFindBody(private->base, private->len)) == NULL) {
 	SMseterror(SMERR_NOBODY, NULL);
-	if (innconf->articlemmap)
+	if (innconf->articlemmap) {
+#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
+	    madvise(private->base, private->len, MADV_DONTNEED);
+#endif
 	    munmap(private->base, private->len);
-	else
-	    free(private->base);
-	free(art->private);
-	free(art);
+	} else {
+	    DISPOSE(private->base);
+	}
+	DISPOSE(art->private);
+	DISPOSE(art);
 	return NULL;
     }
 
     if (amount == RETR_HEAD) {
 	art->data = private->base;
 	art->len = p - private->base;
-        /* Headers end just before the first empty line (\r\n). */
-        art->len = art->len - 2;
 	return art;
     }
 
@@ -317,35 +270,39 @@ static ARTHANDLE *OpenArticle(const char *path, RETRTYPE amount) {
 	return art;
     }
     SMseterror(SMERR_UNDEFINED, "Invalid retrieve request");
-    if (innconf->articlemmap)
+    if (innconf->articlemmap) {
+#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
+	madvise(private->base, private->len, MADV_DONTNEED);
+#endif
 	munmap(private->base, private->len);
-    else
-	free(private->base);
-    free(art->private);
-    free(art);
+    } else {
+	DISPOSE(private->base);
+    }
+    DISPOSE(art->private);
+    DISPOSE(art);
     return NULL;
 }
 
 ARTHANDLE *timehash_retrieve(const TOKEN token, const RETRTYPE amount) {
-    time_t              now;
+    int                 time;
     int                 seqnum;
     char                *path;
     ARTHANDLE           *art;
     static TOKEN	ret_token;
-
+    
     if (token.type != TOKEN_TIMEHASH) {
 	SMseterror(SMERR_INTERNAL, NULL);
 	return NULL;
     }
 
-    BreakToken(token, &now, &seqnum);
-    path = MakePath(now, seqnum, token.class);
+    BreakToken(token, &time, &seqnum);
+    path = MakePath(time, seqnum, token.class);
     if ((art = OpenArticle(path, amount)) != (ARTHANDLE *)NULL) {
-	art->arrived = now;
+	art->arrived = time;
 	ret_token = token;
 	art->token = &ret_token;
     }
-    free(path);
+    DISPOSE(path);
     return art;
 }
 
@@ -354,13 +311,17 @@ void timehash_freearticle(ARTHANDLE *article) {
 
     if (!article)
 	return;
-
+    
     if (article->private) {
 	private = (PRIV_TIMEHASH *)article->private;
-	if (innconf->articlemmap)
+	if (innconf->articlemmap) {
+#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
+	    madvise(private->base, private->len, MADV_DONTNEED);
+#endif
 	    munmap(private->base, private->len);
-	else
-	    free(private->base);
+	} else {
+	    DISPOSE(private->base);
+	}
 	if (private->top)
 	    closedir(private->top);
 	if (private->sec)
@@ -369,55 +330,53 @@ void timehash_freearticle(ARTHANDLE *article) {
 	    closedir(private->ter);
 	if (private->artdir)
 	    closedir(private->artdir);
-	free(private);
+	DISPOSE(private);
     }
-    free(article);
+    DISPOSE(article);
 }
 
-bool timehash_cancel(TOKEN token) {
-    time_t              now;
+BOOL timehash_cancel(TOKEN token) {
+    int                 time;
     int                 seqnum;
     char                *path;
     int                 result;
 
-    BreakToken(token, &now, &seqnum);
-    path = MakePath(now, seqnum, token.class);
+    BreakToken(token, &time, &seqnum);
+    path = MakePath(time, seqnum, token.class);
     result = unlink(path);
-    free(path);
+    DISPOSE(path);
     if (result < 0) {
 	SMseterror(SMERR_UNDEFINED, NULL);
-	return false;
+	return FALSE;
     }
-    return true;
+    return TRUE;
 }
 
 static struct dirent *FindDir(DIR *dir, FINDTYPE type) {
     struct dirent       *de;
-
+    
     while ((de = readdir(dir)) != NULL) {
         if (type == FIND_TOPDIR)
 	    if ((strlen(de->d_name) == 7) &&
 		(strncmp(de->d_name, "time-", 5) == 0) &&
-		isxdigit((unsigned char) de->d_name[5]) &&
-		isxdigit((unsigned char) de->d_name[6]))
+		isxdigit((int)de->d_name[5]) &&
+		isxdigit((int)de->d_name[6]))
 	        return de;
 
 	if (type == FIND_DIR)
-	    if ((strlen(de->d_name) == 2)
-                && isxdigit((unsigned char) de->d_name[0])
-                && isxdigit((unsigned char) de->d_name[1]))
+	    if ((strlen(de->d_name) == 2) && isxdigit((int)de->d_name[0]) && isxdigit((int)de->d_name[1]))
 		return de;
 
 	if (type == FIND_ART)
 	    if ((strlen(de->d_name) == 9) &&
-		isxdigit((unsigned char) de->d_name[0]) &&
-		isxdigit((unsigned char) de->d_name[1]) &&
-		isxdigit((unsigned char) de->d_name[2]) &&
-		isxdigit((unsigned char) de->d_name[3]) &&
-		isxdigit((unsigned char) de->d_name[5]) &&
-		isxdigit((unsigned char) de->d_name[6]) &&
-		isxdigit((unsigned char) de->d_name[7]) &&
-		isxdigit((unsigned char) de->d_name[8]) &&
+		isxdigit((int)de->d_name[0]) &&
+		isxdigit((int)de->d_name[1]) &&
+		isxdigit((int)de->d_name[2]) &&
+		isxdigit((int)de->d_name[3]) &&
+		isxdigit((int)de->d_name[5]) &&
+		isxdigit((int)de->d_name[6]) &&
+		isxdigit((int)de->d_name[7]) &&
+		isxdigit((int)de->d_name[8]) &&
 		(de->d_name[4] == '-'))
 		return de;
 	}
@@ -425,19 +384,15 @@ static struct dirent *FindDir(DIR *dir, FINDTYPE type) {
     return NULL;
 }
 
-ARTHANDLE *
-timehash_next(ARTHANDLE *article, const RETRTYPE amount)
-{
+ARTHANDLE *timehash_next(const ARTHANDLE *article, const RETRTYPE amount) {
     PRIV_TIMEHASH       priv;
     PRIV_TIMEHASH       *newpriv;
     char                *path;
     struct dirent       *de;
     ARTHANDLE           *art;
     int                 seqnum;
-    size_t              length;
 
-    length = strlen(innconf->patharticles) + 32;
-    path = xmalloc(length);
+    path = NEW(char, strlen(innconf->patharticles) + 32);
     if (article == NULL) {
 	priv.top = NULL;
 	priv.sec = NULL;
@@ -448,13 +403,17 @@ timehash_next(ARTHANDLE *article, const RETRTYPE amount)
 	priv.terde = NULL;
     } else {
 	priv = *(PRIV_TIMEHASH *)article->private;
-	free(article->private);
-	free(article);
+	DISPOSE(article->private);
+	DISPOSE((void *)article);
 	if (priv.base != NULL) {
-	    if (innconf->articlemmap)
+	    if (innconf->articlemmap) {
+#if defined(MADV_DONTNEED) && defined(HAVE_MADVISE)
+		madvise(priv.base, priv.len, MADV_DONTNEED);
+#endif
 		munmap(priv.base, priv.len);
-	    else
-		free(priv.base);
+	    } else {
+		DISPOSE(priv.base);
+	    }
 	}
     }
 
@@ -478,45 +437,45 @@ timehash_next(ARTHANDLE *article, const RETRTYPE amount)
 			/* end of search */
 			closedir(priv.top);
 			priv.top = NULL;
-			free(path);
+			DISPOSE(path);
 			return NULL;
 		    }
-		    snprintf(path, length, "%s", innconf->patharticles);
+		    sprintf(path, "%s", innconf->patharticles);
 		    if ((priv.top = opendir(path)) == NULL) {
 			SMseterror(SMERR_UNDEFINED, NULL);
-			free(path);
+			DISPOSE(path);
 			return NULL;
 		    }
 		    if ((priv.topde = FindDir(priv.top, FIND_TOPDIR)) == NULL) {
 			SMseterror(SMERR_UNDEFINED, NULL);
 			closedir(priv.top);
-			free(path);
+			DISPOSE(path);
 			return NULL;
 		    }
 		}
-		snprintf(path, length, "%s/%s", innconf->patharticles, priv.topde->d_name);
+		sprintf(path, "%s/%s", innconf->patharticles, priv.topde->d_name);
 		if ((priv.sec = opendir(path)) == NULL)
 		    continue;
 	    }
-	    snprintf(path, length, "%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name);
+	    sprintf(path, "%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name);
 	    if ((priv.ter = opendir(path)) == NULL)
 		continue;
 	}
-	snprintf(path, length, "%s/%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name, priv.terde->d_name);
+	sprintf(path, "%s/%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name, priv.terde->d_name);
 	if ((priv.artdir = opendir(path)) == NULL)
 	    continue;
     }
     if (de == NULL)
 	return NULL;
-    snprintf(path, length, "%s/%s/%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name, priv.terde->d_name, de->d_name);
+    sprintf(path, "%s/%s/%s/%s/%s", innconf->patharticles, priv.topde->d_name, priv.secde->d_name, priv.terde->d_name, de->d_name);
 
     art = OpenArticle(path, amount);
     if (art == (ARTHANDLE *)NULL) {
-	art = xmalloc(sizeof(ARTHANDLE));
+	art = NEW(ARTHANDLE, 1);
 	art->type = TOKEN_TIMEHASH;
 	art->data = NULL;
 	art->len = 0;
-	art->private = xmalloc(sizeof(PRIV_TIMEHASH));
+	art->private = (void *)NEW(PRIV_TIMEHASH, 1);
 	newpriv = (PRIV_TIMEHASH *)art->private;
 	newpriv->base = NULL;
     }
@@ -528,45 +487,30 @@ timehash_next(ARTHANDLE *article, const RETRTYPE amount)
     newpriv->topde = priv.topde;
     newpriv->secde = priv.secde;
     newpriv->terde = priv.terde;
-    snprintf(path, length, "%s/%s/%s/%s", priv.topde->d_name, priv.secde->d_name, priv.terde->d_name, de->d_name);
+    sprintf(path, "%s/%s/%s/%s", priv.topde->d_name, priv.secde->d_name, priv.terde->d_name, de->d_name);
     art->token = PathToToken(path);
-    BreakToken(*art->token, &(art->arrived), &seqnum);
-    free(path);
+    BreakToken(*art->token, (int *)&(art->arrived), &seqnum);
+    DISPOSE(path);
     return art;
 }
 
-bool timehash_ctl(PROBETYPE type, TOKEN *token UNUSED, void *value) {
+BOOL timehash_ctl(PROBETYPE type, TOKEN *token, void *value) {
     struct artngnum *ann;
 
     switch (type) {
     case SMARTNGNUM:
 	if ((ann = (struct artngnum *)value) == NULL)
-	    return false;
+	    return FALSE;
 	/* make SMprobe() call timehash_retrieve() */
 	ann->artnum = 0;
-	return true;
+	return TRUE;
     default:
-	return false;
+	return FALSE;
     }
 }
 
-bool
-timehash_flushcacheddata(FLUSHTYPE type UNUSED)
-{
-    return true;
-}
-
-void
-timehash_printfiles(FILE *file, TOKEN token, char **xref UNUSED,
-                    int ngroups UNUSED)
-{
-    time_t now;
-    int seqnum;
-    char *path;
-
-    BreakToken(token, &now, &seqnum);
-    path = MakePath(now, seqnum, token.class);
-    fprintf(file, "%s\n", path);
+BOOL timehash_flushcacheddata(FLUSHTYPE type) {
+    return TRUE;
 }
 
 void timehash_shutdown(void) {
